
#include "ContextOTFunctionViewer.h"



contextOTFunctionViewer::contextOTFunctionViewer(QWidget *parent, const char *name) : QWidget(parent, name)
{
	this->setBackgroundColor(QColor(245, 235, 235));
	resetOpacityFunction();

    for (int i = 0; i < 256; i++)
    {
		histogram[i] = 0;
    }
}


contextOTFunctionViewer::~contextOTFunctionViewer()
{
}


bool contextOTFunctionViewer::getChanged()
{
	return changed;
}


void contextOTFunctionViewer::resetChanged()
{
	changed = false;
}


void contextOTFunctionViewer::setHistogram(unsigned int *histo)
{
	unsigned int max = 0;
	int i;
	double factor;
	for (i = 1; i < 256; i++)
	{
		if (max < histo[i])
			max = histo[i];
	}
	for (i = 0; i < 256; i++)
	{
		if (histo[i] != 0)
		{
			factor = sqrt(max/histo[i]);
			histogram[i] = (unsigned int)(histo[i]*factor);
		}
    }
	update();
}


void contextOTFunctionViewer::resetOpacityFunction()
{
	opacityFunction[0].gradient = 0;
	opacityFunction[0].pos = 0;
	opacityFunction[1].gradient = 0.5;
	opacityFunction[1].pos = 0.1;
	opacityFunction[2].gradient = 0.5;
	opacityFunction[2].pos = 1;
	points = 3;
	mousePressed = false;
	movingX = true;
	changed = false;
	update();
}


void contextOTFunctionViewer::setOpacityFunction(vtkPiecewiseFunction *piecewiseFunction, float range)
{
	int length = piecewiseFunction->GetSize();
	if (length < 20)
	{
		float *data = piecewiseFunction->GetDataPointer();
		for (int i = 0; i < length; i++)
		{
			opacityFunction[i].pos = data[2*i]/range;
			opacityFunction[i].gradient = data[2*i+1]*4;
		}
		points = length;
		mousePressed = false;
		movingX = true;
		update();
	}
}


void contextOTFunctionViewer::getOpacityFunction(vtkPiecewiseFunction *piecewiseFunction, float range)
{
	for (int i = 0; i < points; i++)
	{
		piecewiseFunction->AddPoint(opacityFunction[i].pos*range, opacityFunction[i].gradient/4);
	}
}


void contextOTFunctionViewer::deletePoint(int index)
{
	for (int i = index; i < points - 1; i++)
	{
		opacityFunction[i] = opacityFunction[i+1];
	}
	points = points - 1;
}


void contextOTFunctionViewer::insertPoint(float x, float y)
{
	for (int i = 0; i < points; i++)
	{
		if (opacityFunction[i].pos > x)
		{
			for (int j = points; j > i; j--)
			{
				opacityFunction[j].pos = opacityFunction[j-1].pos;
				opacityFunction[j].gradient = opacityFunction[j-1].gradient;
			}
			opacityFunction[i].pos = x;
			opacityFunction[i].gradient = y;
			i = points;
		}
	}
	points = points + 1;
}


int contextOTFunctionViewer::testIntersection(int x, int y, int index)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	for (int i = 0; i < points; i++)
	{
		if (i != index)
		{
			int xPos = (int)(opacityFunction[i].pos * factorX) + 3;
			int yPos = height - (int)(opacityFunction[i].gradient * factorY) - 3;
			if (x > xPos-3 && x < xPos+3 && y > yPos-3 && y < yPos+3)
				return i;
		}
	}
	return -1;
}


void contextOTFunctionViewer::paintHistogram(QPainter *paint)
{
    int i;
    unsigned int max = 0;
    double stepx = (double)this->width()/(double)256;
    QPen *pen = new QPen(QColor (150, 150, 150));
    paint->setPen(*pen);
    for (i = 1; i < 256; i++)
    {
		if (max < histogram[i])
			max = histogram[i];
    }
    histogram[0] = 0;
    double stepy = (double)this->height()/(double)(max * 1.1);
    double x = 0;
    int ymax = this->height() - 1;
    for (i = 1; i  < 256; i++)
	{	
		paint->drawLine ((int)(x), ymax-(int)(histogram[i-1]*stepy), (int)(x+stepx), ymax-(int)(histogram[i]*stepy));
		x = x + stepx;
    }
    delete pen;
}


void contextOTFunctionViewer::paintCubes(QPainter *paint)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	QPen *pen = new QPen(QColor (0, 0, 0));
    paint->setPen(*pen);
	for (int i = 0; i < points; i++)
	{
		int xPos = (int)(opacityFunction[i].pos * factorX) + 3;
		int yPos = height - (int)(opacityFunction[i].gradient * factorY) - 3;
		paint->drawLine(xPos-3, yPos-3, xPos+3, yPos-3);
		paint->drawLine(xPos+3, yPos-3, xPos+3, yPos+3);
		paint->drawLine(xPos+3, yPos+3, xPos-3, yPos+3);
		paint->drawLine(xPos-3, yPos+3, xPos-3, yPos-3);
	}
	delete pen;
}


void contextOTFunctionViewer::paintLines(QPainter *paint)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	QPen *pen = new QPen(QColor (0, 0, 0));
    paint->setPen(*pen);
	for (int i = 1; i < points; i++)
	{
		int xPos1 = (int)(opacityFunction[i-1].pos * factorX) + 3;
		int yPos1 = height - (int)(opacityFunction[i-1].gradient * factorY) - 3;
		int xPos2 = (int)(opacityFunction[i].pos * factorX) + 3;
		int yPos2 = height - (int)(opacityFunction[i].gradient * factorY) - 3;
		paint->drawLine(xPos1, yPos1, xPos2, yPos2);
	}
	delete pen;
}


void contextOTFunctionViewer::mousePressEvent(QMouseEvent *event)
{
	mousePressed = true;
	intersectionIndex = testIntersection(event->x(), event->y(), -1);
	if (intersectionIndex == 0 || intersectionIndex == points-1)
		movingX = false;
}


void contextOTFunctionViewer::mouseReleaseEvent(QMouseEvent *event)
{
	mousePressed = false;
	movingX = true;
}


void contextOTFunctionViewer::mouseDoubleClickEvent(QMouseEvent *event)
{
	if (points < 20)
	{
		int width = this->width();
		int height = this->height();
		int x = event->x();
		int y = height - event->y();
		if (x > 3 && x < width-4 && y > 3 && y < height-4)
		{
			double factorX = width - 6;
			double factorY = height - 6;
			int intersect = testIntersection(x, y, -1);
			if (intersect < 0)
			{
				insertPoint((float)((x-3)/factorX), (float)((y-3)/factorY));
				changed = true;
				update();
			}
		}
	}
}


void contextOTFunctionViewer::mouseMoveEvent(QMouseEvent *event)
{
	if (mousePressed == true && intersectionIndex >= 0)
	{
		int width = this->width();
		int height = this->height();
		double factorX = width - 6;
		double factorY = height - 6;
		int x = event->x() - 3;
		int y = height - event->y() - 3;
		float xPos = (float)((x)/factorX);
		float yPos = (float)((y)/factorY);
		if (yPos < 0)
			yPos = 0;
		if (yPos > 1)
			yPos = 1;
		if (intersectionIndex != 0 && intersectionIndex != points-1)
		{
			if (x > (int)(opacityFunction[intersectionIndex-1].pos * factorX)+1 && x < (int)(opacityFunction[intersectionIndex+1].pos * factorX)-1)
				movingX = true;
			int intersect = testIntersection(event->x(), event->y(), intersectionIndex);
			if (intersect >= 0)
			{
				deletePoint(intersectionIndex);
				update();
				movingX = false;
				intersectionIndex = -1;
			}
		}
		if (movingX == false && intersectionIndex >= 0)
		{
			opacityFunction[intersectionIndex].gradient = yPos;
			changed = true;
			update();
		}
		else if (movingX == true && intersectionIndex >= 0)
		{
			if (x <= (int)(opacityFunction[intersectionIndex-1].pos * factorX)+1 || x >= (int)(opacityFunction[intersectionIndex+1].pos * factorX)-1)
				movingX = false;
			else
			{
				opacityFunction[intersectionIndex].pos = xPos;
				opacityFunction[intersectionIndex].gradient = yPos;
				changed = true;
				update();
			}
		}
	}
}


void contextOTFunctionViewer::paintEvent(QPaintEvent *event)
{
	QPainter *paint = new QPainter(this);
	paintHistogram(paint);
	paintCubes(paint);
	paintLines(paint);
	delete paint;
}