#ifndef DOFANIMATIONEDITORIMPL_H
#define DOFANIMATIONEDITORIMPL_H


#include <qwidget.h>

#include "DOFAnimationEditor.h"


//! Class DOFAnimationEditorImpl
/*!
    Die Klasse ist abgeleitet von DOFAnimationEditor und ist f&uuml;r die Abl&auml;ufe
	im Depth of Field Animationsfenster zust&auml;ndig.
*/
class DOFAnimationEditorImpl : public DOFAnimationEditor
{
    Q_OBJECT
     
 public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen. dim, orient und focPoint
		geben die Gr&ouml;sse, die Orientierung des Datensatzes und den Fokuspunkt der Kamera an.
	*/
    DOFAnimationEditorImpl(QWidget *parent, const char* name, int dim [3], float orient [3], double focPoint [3]);
    //! Methode getStart
	/*!
		Gibt die Startdicke der Linse zur&uuml;ck.
	*/
	float getStart();
	//! Methode getEnd
	/*!
		Gibt die Enddicke der Linse zur&uuml;ck.
	*/
    float getEnd();
	//! Methode getFrames
	/*!
		Gibt die Anzahl der Frames der Animation zur&uuml;ck.
	*/
    int getFrames();
	//! Methode getSamplePoints
	/*!
		Gibt die Anzahl der Sample-Punkte zur&uuml;ck.
	*/
	int getSamplePoints();
	//! Methode getFixedPoint
	/*!
		Ist true, falls sich der Fokuspunkt nur in z-Richtung ver&auml;ndern soll.
	*/
	bool getFixedPoint();
    
    
	/*!
		Variable speichert die Anzahl der Animationspunkte.
	*/
	int numberPoints;
	/*!
		Array mit den Animationspunkten.
	*/
    double focalArray [20][3];
	/*!
		Wenn diese Variable true ist, kann gerendert werden.
	*/
    bool rendering;
    
 public slots:
	//! Slot cancelPressed
	/*!
		Wenn Cancel Button geclickt wird.
	*/
    void cancelPressed();
	//! Slot startPressed
	/*!
		Wird aufgerufen, wenn im Dialog auf Animationsstart geclickt wird.
	*/
    void startPressed();
	//! Slot addPointPressed
	/*!
		Wird aufgerufen, wenn im Dialog auf Animationspunkt hinzuf&uuml;gen geclickt wird.
	*/
    void addPointPressed();
	//! Slot deletePointPressed
	/*!
		Wird aufgerufen, wenn im Dialog auf Animationspunkt l&ouml;schen geclickt wird.
	*/
    void deletePointPressed();
	//! Slot startChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r die Anfangsdicke der Linse ver&auml;ndert wird.
	*/
    void startChanged(int value);
	//! Slot endChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r die Enddicke der Linse ver&auml;ndert wird.
	*/
    void endChanged(int value);
	//! Slot framesChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r die Anzahl der Frames ver&auml;ndert wird.
	*/
    void framesChanged(int value);
	//! Slot samplePointsChanged
	/*!
		Wird aufgerufen, wenn der Slider f&uuml;r die Anzahl der Sample-Punkte ver&auml;ndert wird.
	*/
	void samplePointsChanged(int value);
 
 private:
    int dimension [3];
    double focalPoint [3];
    float orientation [3];
    float start, end;
    int frames;
	int samplePoints;
};


#endif  // DOFANIMATIONEDITORIMPL
