#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>


#include "DithAnim_newPointImpl.h"


#define PI 3.14159265


DithAnim_newPointImpl::DithAnim_newPointImpl(QWidget *parent, const char *name, DithAnim_struct newPoint, DithAnim_struct oldPoint) : DithAnim_newPoint(parent, name)
{
	anim = newPoint;
	animRef = oldPoint;
	setValues();
	exitStatus = false;
}


DithAnim_newPointImpl::~DithAnim_newPointImpl()
{
}


DithAnim_struct DithAnim_newPointImpl::getDithAnimStruct()
{
	return anim;
}


bool DithAnim_newPointImpl::getExitStatus()
{
	return exitStatus;
}


bool DithAnim_newPointImpl::getValues()
{
	anim.distance = (float)distanceSlider->value()/100;
	anim.size = (float)sizeSlider->value()/100;
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	anim.xClockwise = xRotClockwise->isChecked();
	anim.yClockwise = yRotClockwise->isChecked();
	anim.zClockwise = zRotClockwise->isChecked();

	anim.samples = samplesSlider->value();

	return true;
}


void DithAnim_newPointImpl::setValues()
{
	// Dithering Settings
	distanceSlider->setValue((int)(anim.distance*100));
	sizeSlider->setValue((int)(anim.size*100));
	distanceLabel->setText(QString::number(anim.distance));
	sizeLabel->setText(QString::number(anim.size));

	// Rotation
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));

	xRotClockwise->setChecked(anim.xClockwise);
	yRotClockwise->setChecked(anim.yClockwise);
	zRotClockwise->setChecked(anim.zClockwise);
	
	int diffX, diffY, diffZ;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;

	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;

	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;

	xRotLabel_diff->setText(QString::number(diffX));
	yRotLabel_diff->setText(QString::number(diffY));
	zRotLabel_diff->setText(QString::number(diffZ));

	samplesSlider->setValue(anim.samples);
	samplesLabel->setText(QString::number(anim.samples));
}


void DithAnim_newPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void DithAnim_newPointImpl::distanceSliderChanged(int value)
{
	distanceLabel->setText(QString::number((float)value/100));
	anim.distance = (float)value/100;
}


void DithAnim_newPointImpl::sizeSliderChanged(int value)
{
	sizeLabel->setText(QString::number((float)value/100));
	anim.size = (float)value/100;
}


void DithAnim_newPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void DithAnim_newPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void DithAnim_newPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}


void DithAnim_newPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}


void DithAnim_newPointImpl::samplePointsSliderChanged(int value)
{
	samplesLabel->setText(QString::number(value));
	anim.samples = value;
}


void DithAnim_newPointImpl::xRotClockwiseClicked(bool status)
{
	anim.xClockwise = status;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void DithAnim_newPointImpl::yRotClockwiseClicked(bool status)
{
	anim.yClockwise = status;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void DithAnim_newPointImpl::zRotClockwiseClicked(bool status)
{
	anim.zClockwise = status;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}