#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>


#include "DithAnim_startPointImpl.h"


#define PI 3.14159265


DithAnim_startPointImpl::DithAnim_startPointImpl(QWidget *parent, const char *name, DithAnim_struct point) : DithAnim_startPoint(parent, name)
{
	anim = point;
	setValues();
	exitStatus = false;
}


DithAnim_startPointImpl::~DithAnim_startPointImpl()
{
}


DithAnim_struct DithAnim_startPointImpl::getDithAnimStruct()
{
	return anim;
}


bool DithAnim_startPointImpl::getExitStatus()
{
	return exitStatus;
}


bool DithAnim_startPointImpl::getValues()
{
	anim.distance = (float)distanceSlider->value()/100;
	anim.size = (float)sizeSlider->value()/100;

	anim.context = context->isChecked();
	anim.structure = structure->isChecked();
	anim.sec_row = sec_row->isChecked();
	anim.sec_row_structure = false;
	if (anim.sec_row)
		anim.sec_row_structure = sec_row_structure->isChecked();
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	return true;
}


void DithAnim_startPointImpl::setValues()
{
	distanceSlider->setValue((int)(anim.distance*100));
	sizeSlider->setValue((int)(anim.size*100));
	distanceLabel->setText(QString::number(anim.distance));
	sizeLabel->setText(QString::number(anim.size));
	context->setChecked(anim.context);
	structure->setChecked(anim.structure);
	sec_row->setChecked(anim.sec_row);
	sec_row_structure->setChecked(anim.sec_row_structure);

	if (sec_row->isChecked())
		sec_row_structure->setEnabled(true);
	else
		sec_row_structure->setEnabled(false);

	// Slider
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));
}


void DithAnim_startPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void DithAnim_startPointImpl::distanceSliderChanged(int value)
{
	distanceLabel->setText(QString::number((float)value/100));
	anim.distance = (float)value/100;
}


void DithAnim_startPointImpl::sizeSliderChanged(int value)
{
	sizeLabel->setText(QString::number((float)value/100));
	anim.size = (float)value/100;
}


void DithAnim_startPointImpl::secRowClicked(bool status)
{
	if (status)
		sec_row_structure->setEnabled(true);
	else
		sec_row_structure->setEnabled(false);
}


void DithAnim_startPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
}


void DithAnim_startPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
}


void DithAnim_startPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
}


void DithAnim_startPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}
