#ifndef FCANIM_NEWPOINTIMPL_h
#define FCANIM_NEWPOINTIMPL_h

#include <qwidget.h>

#include "FCAnim_newPoint.h"
#include "FCAnim_struct.h"


//! Class FCAnim_newPointImpl
/*!
    Diese Klasse ist abgeleitet von der Klasse FCAnim_newPoint. In ihr sind die 
	Methoden fr die Actions im Dialog ausprogrammiert.
*/
class FCAnim_newPointImpl : public FCAnim_newPoint
{
    Q_OBJECT;
    
public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen. newPoint ist eine Struktur, 
		welche die Werte des aktuellen Animations Punkts enth&auml;t. oldPoint enth&auml;lt die
		Werte des Vorg&auml;nger.
	*/
    FCAnim_newPointImpl(QWidget* parent, const char* name, FCAnim_struct newPoint, FCAnim_struct oldPoint);
    //! Destruktor
	~FCAnim_newPointImpl();
	//! Methode getExitStatus
	/*!
		Gibt true zur&uuml;ck, wenn der Dialog mit OK beendet wurde.
	*/
	bool getExitStatus();
	//! Methode getFCAnimStruct
	/*!
		Gibt die neuen Werte des Animationspunkts in einer Struktur zur&uuml;ck.
	*/
	FCAnim_struct getFCAnimStruct();
    
public slots:
    //! Slot xRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die x-Rotation bewegt wird.
	*/
    void xRotSliderChanged(int value);
	//! Slot yRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die y-Rotation bewegt wird.
	*/
	void yRotSliderChanged(int value);
	//! Slot zRotSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die z-Rotation bewegt wird.
	*/
	void zRotSliderChanged(int value);
	//! Slot zoomSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r den Zoom bewegt wird.
	*/
	void zoomSliderChanged(int value);
	//! Slot okayButtonClicked
	/*!
		Wird aufgerufen wenn der OK Button geclickt wird.
	*/
	void okayButtonClicked();
	//! Slot setFCRegionButtonClicked
	/*!
		Wird aufgerufen wenn der Focus+Context Bereichs Button geclickt wird.
	*/
	void setFCRegionButtonClicked();
	//! Slot samplePointsSliderChanged
	/*!
		Wird aufgerufen wenn der Slider f&uuml;r die Anzahl der Frames bewegt wird.
	*/
	void samplePointsSliderChanged(int value);
	//! Slot xRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die x-Achse
		geclickt wird.
	*/
	void xRotClockwiseClicked(bool status);
	//! Slot yRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die y-Achse
		geclickt wird.
	*/
	void yRotClockwiseClicked(bool status);
	//! Slot zRotClockwiseClicked
	/*!
		Wird aufgerufen wenn die Check Box f&uuml;r die Rotationsrichtung um die z-Achse
		geclickt wird.
	*/
	void zRotClockwiseClicked(bool status);
    
private:
	bool getValues();
	void setValues();

	FCAnim_struct anim;
	FCAnim_struct animRef;
	bool exitStatus;
};


#endif