#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>


#include "FCAnim_startPointImpl.h"
#include "focusContextEditorImpl.h"


#define PI 3.14159265


FCAnim_startPointImpl::FCAnim_startPointImpl(QWidget *parent, const char *name, FCAnim_struct point) : FCAnim_startPoint(parent, name)
{
	anim = point;
	setValues();
	exitStatus = false;
}


FCAnim_startPointImpl::~FCAnim_startPointImpl()
{
}


FCAnim_struct FCAnim_startPointImpl::getFCAnimStruct()
{
	return anim;
}


bool FCAnim_startPointImpl::getExitStatus()
{
	return exitStatus;
}


bool FCAnim_startPointImpl::getValues()
{
	bool flag;
	anim.center[0] = (centerX->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.center[1] = (centerY->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.center[2] = (centerZ->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.radius = (regionRadius->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.transfC = (FC_transferCoefficient->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.ksc = (FC_ksc->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.kss = (FC_kss->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.kse = (FC_kse->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.redOffset = (FC_redOffset->text()).toFloat(&flag);
	if (!flag)
		return false;
	anim.redFactor = (FC_redFactor->text()).toFloat(&flag);
	if (!flag)
		return false;
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	return true;
}


void FCAnim_startPointImpl::setValues()
{
	centerX->setText(QString::number(anim.center[0]));
	centerY->setText(QString::number(anim.center[1]));
	centerZ->setText(QString::number(anim.center[2]));
	regionRadius->setText(QString::number(anim.radius));
	FC_transferCoefficient->setText(QString::number(anim.transfC));
	FC_ksc->setText(QString::number(anim.ksc));
	FC_kss->setText(QString::number(anim.kss));
	FC_kse->setText(QString::number(anim.kse));
	FC_redOffset->setText(QString::number(anim.redOffset));
	FC_redFactor->setText(QString::number(anim.redFactor));

	// Slider
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));
}


void FCAnim_startPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void FCAnim_startPointImpl::setFCRegionButtonClicked()
{
    float middle [3];
	float radius;
	bool flag, error1 = false, error2 = false;
	middle[0] = (centerX->text()).toFloat(&flag);
	if (!flag)
		error1 = true;
	middle[1] = (centerY->text()).toFloat(&flag);
	if (!flag)
		error1 = true;
	middle[2] = (centerZ->text()).toFloat(&flag);
	if (!flag)
		error1 = true;
	radius = (regionRadius->text()).toFloat(&flag);
	if (!flag)
		error2 = true;
	if (error1)
	{
		middle[0] = anim.center[0];
		middle[1] = anim.center[1];
		middle[2] = anim.center[2];
	}
	if (error2)
	{
		radius = anim.radius;
	}
	float orientation [3];
	orientation[0] = (float)anim.xRot;
	orientation[1] = (float)anim.yRot;
	orientation[2] = (float)anim.zRot;
	focusContextEditorImpl *focConEditor = new focusContextEditorImpl(this, "Renderer - Set Focus Region", anim.dimension, orientation, (double *)anim.focPoint, middle, radius);
    focConEditor->exec();
    if (focConEditor->okay)
    {
		float *middle;
		middle = focConEditor->getFocalPoint();
		anim.radius = focConEditor->getRadius();
		anim.center[0] = middle[0];
		anim.center[1] = middle[1];
		anim.center[2] = middle[2];
		centerX->setText(QString::number(anim.center[0]));
		centerY->setText(QString::number(anim.center[1]));
		centerZ->setText(QString::number(anim.center[2]));
		regionRadius->setText(QString::number(anim.radius));
    }
	delete focConEditor;
}


void FCAnim_startPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
}


void FCAnim_startPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
}


void FCAnim_startPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
}


void FCAnim_startPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}
