#include <qlistbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>

#include "FCAnimationEditorImpl.h"
#include "FCAnim_startPointImpl.h"
#include "FCAnim_newPointImpl.h"



FCAnimationEditorImpl::FCAnimationEditorImpl(QWidget* parent, const char* name, FCAnim_struct startPoint) : FCAnimationEditor(parent, name)
{
	exitStatus = false;
	anim = new FCAnim_struct [30];
	anim[0] = startPoint;
	number = 1;
	frames = 0;
	frameCounter->setText("0");
}


FCAnimationEditorImpl::~FCAnimationEditorImpl()
{
}


bool FCAnimationEditorImpl::getExitStatus()
{
	return exitStatus;
}


FCAnim_struct *FCAnimationEditorImpl::getFCAnimStruct()
{
	return anim;
}


int FCAnimationEditorImpl::getNumbers()
{
	return number;
}


void FCAnimationEditorImpl::deletePointButtonClicked()
{
	int pos = pointBox->currentItem();
	if (pos >= 0)
	{
		for (int i = pos+1; i < number-1; i++)
		{
			anim[i] = anim[i+1];
		}
		number = number-1;
		pointBox->removeItem(pos);
		if (pos+1 < number)
			pointBox->setCurrentItem(pos);
		else
			pointBox->setCurrentItem(pos-1);
		frames = 0;
		for (i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
	}
}


void FCAnimationEditorImpl::editPointButtonClicked()
{
	int pos = pointBox->currentItem()+1;
	FCAnim_newPointImpl *editPoint = new FCAnim_newPointImpl(this, "Edit Point", anim[pos], anim[pos-1]);
	editPoint->exec();
	if (editPoint->getExitStatus())
	{
		anim[pos] = editPoint->getFCAnimStruct();
		frames = 0;
		for (int i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
		QString frameString = "[" + QString::number(anim[pos].samples) + " frames]";
		pointBox->changeItem(frameString, pos-1);
	}
	delete editPoint;
}


void FCAnimationEditorImpl::newPointButtonClicked()
{
	if (number <= 30)
	{
		FCAnim_newPointImpl *newPoint = new FCAnim_newPointImpl(this, "Edit Point", anim[number-1], anim[number-1]);
		newPoint->exec();
		if (newPoint->getExitStatus())
		{
			anim[number] = newPoint->getFCAnimStruct();
			number = number+1;
			QString frameString = "[" + QString::number(anim[number-1].samples) + " frames]";
			pointBox->insertItem(frameString, -1);
			frames = 0;
			for (int i = 1; i < number; i++)
				frames += anim[i].samples; 
			frameCounter->setText(QString::number(frames));
		}
		delete newPoint;
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "Maximum Point Count is reached");
}


void FCAnimationEditorImpl::startPointButtonClicked()
{
	FCAnim_startPointImpl *startPoint = new FCAnim_startPointImpl(this, "Edit Point", anim[0]);
	startPoint->exec();
	if (startPoint->getExitStatus())
	{
		anim[0] = startPoint->getFCAnimStruct();
	}
	delete startPoint;
}



void FCAnimationEditorImpl::startAnimationButtonClicked()
{
	if (number > 1)
		exitStatus = true;
	close();
}