
#include <qstring.h>
#include <qpen.h>
#include <math.h>
#include <iostream.h>

#include "focalWidget.h"


#define PI 3.14159265


focalWidget::focalWidget (QWidget *parent, const char *name) : QWidget (parent)
{
    this->setPaletteBackgroundColor (QColor (255, 255, 255));
    dim = new int [3];
    orientation = new float [3];
    focalPoint = new double [3];
    dev_focalPoint = new int [3];
    focalPointSet = false;
}


focalWidget::~focalWidget ()
{
}


void focalWidget::setView (int *dimension, float *orient, double *focPoint, QString direction)
{
    for (int i = 0; i < 3; i++)
    {
		orientation[i] = orient[i];
		focalPoint[i] = focPoint[i];
		dim[i] = dimension[i];
    }
	if (direction.compare("front") == 0)
		viewDirection = 0;
    else
		viewDirection = 1;
	//cout << "focal Point: " << focalPoint[0] << ", " << focalPoint[1] << ", " << focalPoint[2] << "\n";
    getPoints();
    getFocalPointPos();
    update();
}


double focalWidget::getDevFactor()
{
    int x = this->width();
    int y = this->height();
    int max = 0;
    for (int i = 0; i < 3; i++)
    {
		if (dim[i] > max)
			max = dim[i];
    }
    double factor;
    if (x < y)
    {
		factor = (double)x/(2*(double)max);
    }
    else
    {
		factor = (double)y/(2*(double)max);
    }
    return factor;
}


void focalWidget::paintCube(QPainter *painter)
{
    QPen *pen = new QPen(QColor(0, 0, 0));
    painter->setPen(*pen);
    int dev_points[8][3];
    int x = this->width();
    int y = this->height();
	int i;

    double factor = getDevFactor();
    for (i = 0; i < 8; i++)
    {
		dev_points[i][0] = (int)(points[i][0] * factor) + (int)(x/2);
		if (viewDirection == 0)
		{
			dev_points[i][1] = (int)(-points[i][1] * factor) + (int)(y/2);
			dev_points[i][2] = (int)(points[i][2] * factor) + (int)(y/2);
		}
		else
		{
			dev_points[i][1] = (int)(points[i][1] * factor) + (int)(y/2);
			dev_points[i][2] = (int)(points[i][2] * factor) + (int)(y/2);
		}
    }
   
    
    int depth, view;
    if (viewDirection == 0)
    {
		depth = 2;
		view = 1;
    }
    else
    {
		depth = 1;
		view = 2;
    }
    int minimum = 0;
    double min = dev_points[0][depth];
    for (i = 1; i < 8; i++)
    {
		if (dev_points[i][depth] < min)
		{
			minimum = i;
			min = dev_points[i][depth];
		}
    }
    for (i = 0; i < 4; i++)
    {
		int j = (i+1)%4;
		int k = i+4;
		int l = 4+j;
		if (i != minimum && j != minimum)
		{
			painter->drawLine(dev_points[i][0], dev_points[i][view], dev_points[j][0], dev_points[j][view]);
		}
		if (i != minimum && k != minimum)
		{
			painter->drawLine(dev_points[i][0], dev_points[i][view], dev_points[k][0], dev_points[k][view]);
		}
		if (k != minimum && l != minimum)
		{
			painter->drawLine(dev_points[k][0], dev_points[k][view], dev_points[l][0], dev_points[l][view]);
		}
    }
    delete pen;
}


void focalWidget::paintFocalPoint(QPainter *painter)
{
    if (!focalPointSet)
    {
		getFocalPointPos ();
		focalPointSet = true;
    }
    
    QPen *pen = new QPen (QColor ("red"), 2);
    painter->setPen (*pen);
    
    
    //cout << "dev Point: " << this->width() << ", " << this->height() << "\n";
    
    
    if (viewDirection == 0)
    {
		painter->drawLine (dev_focalPoint[0]-10, dev_focalPoint[1], dev_focalPoint[0]+10, dev_focalPoint[1]);
		painter->drawLine (dev_focalPoint[0], dev_focalPoint[1]-10, dev_focalPoint[0], dev_focalPoint[1]+10);
    }
    else
    {
		painter->drawLine (dev_focalPoint[0]-10, dev_focalPoint[2], dev_focalPoint[0]+10, dev_focalPoint[2]);
		painter->drawLine (dev_focalPoint[0], dev_focalPoint[2]-10, dev_focalPoint[0], dev_focalPoint[2]+10);
    }
    delete pen;
}
    

void focalWidget::rotatePoint(double *point, float *rotation)
{
	double new_point [3];

	new_point[1] = point[1]*cos((double)rotation[0]*PI/180) - point[2]*sin((double)rotation[0]*PI/180);
	new_point[2] = point[1]*sin((double)rotation[0]*PI/180) + point[2]*cos((double)rotation[0]*PI/180);	
	point[1] = new_point[1];
	point[2] = new_point[2];

	new_point[0] = point[2]*sin((double)rotation[1]*PI/180) + point[0]*cos((double)rotation[1]*PI/180);
	new_point[2] = point[2]*cos((double)rotation[1]*PI/180) - point[0]*sin((double)rotation[1]*PI/180);
	point[0] = new_point[0];
	point[2] = new_point[2];

	new_point[0] = point[0]*cos((double)rotation[2]*PI/180) - point[1]*sin((double)rotation[2]*PI/180);
	new_point[1] = point[0]*sin((double)rotation[2]*PI/180) + point[1]*cos((double)rotation[2]*PI/180);
	point[0] = new_point[0];
	point[1] = new_point[1];
}



void focalWidget::getPoints()
{
    double help [3];
	int i;

    help[0] = (double)dim[0]/2;
    help[1] = (double)dim[1]/2;
    help[2] = (double)dim[2]/2;
    for (i = 0; i < 8; i++)
    {
		points[i][0] = help[0];
		points[i][1] = help[1];
		if (i < 4)
			points[i][2] = help[2];
		else
			points[i][2] = -help[2];
    }
    points[0][0] = -points[0][0];
    points[3][0] = -points[3][0];
    points[4][0] = -points[4][0];
    points[7][0] = -points[7][0];
    
    points[2][1] = -points[2][1];
    points[3][1] = -points[3][1];
    points[6][1] = -points[6][1];
    points[7][1] = -points[7][1];
    

    for (i = 0; i < 8; i++)
	{
		rotatePoint(points[i], orientation);
    }
}


void focalWidget::getFocalPointPos()
{    
    // get Device Coordinates
    
    double factor = getDevFactor();
    int x = this->width();
    int y = this->height();
    dev_focalPoint[0] = (int)(focalPoint[0] * factor) + (int)(x/2);
    if (viewDirection == 0)
    {
		dev_focalPoint[1] = (int)(-focalPoint[1] * factor) + (int)(y/2);
		dev_focalPoint[2] = (int)(focalPoint[2] * factor) + (int)(y/2);
    }
    else
    {
		dev_focalPoint[1] = (int)(-focalPoint[1] * factor) + (int)(y/2);
		dev_focalPoint[2] = (int)(focalPoint[2] * factor) + (int)(y/2);
    }
}


double* focalWidget::getFocalPoint()
{
    double *help = new double [3];
    int x = this->width();
    int y = this->height();
    
    double factor = getDevFactor();
    
    help[0] = ((double)dev_focalPoint[0] - (double)(x/2))/factor;
    help[1] = ((double)dev_focalPoint[1] - (double)(y/2))/factor;
    help[2] = ((double)dev_focalPoint[2] - (double)(y/2))/factor;
    help[1] = help[1]*(-1);
    
    return help;
}


void focalWidget::mousePressEvent(QMouseEvent *event)
{
    dev_focalPoint[0] = event->x();
    if (viewDirection == 0)
    {
		dev_focalPoint[1] = event->y();
    }
    else
    {
		dev_focalPoint[2] = event->y();
    }
    update();
    changeFocalPoint(dev_focalPoint);
}


void focalWidget::updateFocalPoint(int *point)
{
    dev_focalPoint = point;
    update();
}


void focalWidget::paintEvent(QPaintEvent *event)
{
    QPainter *painter = new QPainter(this);
    paintCube(painter);
    paintFocalPoint(painter);
    delete painter;
}
