
#include <qstring.h>
#include <qpen.h>
#include <math.h>
#include <iostream.h>

#include "FocusContextWidget.h"


#define PI 3.14159265


focusContextWidget::focusContextWidget (QWidget *parent, const char *name) : QWidget (parent)
{
    this->setPaletteBackgroundColor (QColor (255, 255, 255));
    dim = new int [3];
    orientation = new float [3];
    focalPoint = new float [3];
	camFocPoint = new float [3];
    dev_focalPoint = new int [3];
    focalPointSet = false;
}


focusContextWidget::~focusContextWidget ()
{
}


void focusContextWidget::setView (int *dimension, float *orient, float *focPoint, float *middle, float rad, QString direction)
{
    float *rot = new float [3];
	for (int i = 0; i < 3; i++)
    {
		orientation[i] = orient[i];
		rot[i] = orient[i];
		focalPoint[i] = middle[i];
		camFocPoint[i] = focPoint[i];
		dim[i] = dimension[i];
    }
	if (direction.compare("front") == 0)
		viewDirection = 0;
    else
		viewDirection = 1;
	radius = rad;
	//rot[1] = -rot[1];
	//middle[2] = -middle[2];
	rotatePoint(focalPoint, rot);

    getPoints();
    getFocalPointPos();
    update();
}


void focusContextWidget::setCameraFocalPoint()
{
	for (int i = 0; i < 3; i++)
		focalPoint[i] = camFocPoint[i];
	getFocalPointPos();
	update();
}


double focusContextWidget::getDevFactor()
{
    int x = this->width();
    int y = this->height();
    int max = 0;
    for (int i = 0; i < 3; i++)
    {
		if (dim[i] > max)
			max = dim[i];
    }
    double factor;
    if (x < y)
    {
		factor = (double)x/(2*(double)max);
    }
    else
    {
		factor = (double)y/(2*(double)max);
    }
    return factor;
}


void focusContextWidget::paintCube(QPainter *painter)
{
    QPen *pen = new QPen(QColor(0, 0, 0));
    painter->setPen(*pen);
    int dev_points[8][3];
    int x = this->width();
    int y = this->height();
	int i;

    double factor = getDevFactor();
    for (i = 0; i < 8; i++)
    {
		dev_points[i][0] = (int)(points[i][0] * factor) + (int)(x/2);
		if (viewDirection == 0)
		{
			dev_points[i][1] = (int)(-points[i][1] * factor) + (int)(y/2);
			dev_points[i][2] = (int)(points[i][2] * factor) + (int)(y/2);
		}
		else
		{
			dev_points[i][1] = (int)(points[i][1] * factor) + (int)(y/2);
			dev_points[i][2] = (int)(points[i][2] * factor) + (int)(y/2);
		}
    }
   
    
    int depth, view;
    if (viewDirection == 0)
    {
		depth = 2;
		view = 1;
    }
    else
    {
		depth = 1;
		view = 2;
    }
    int minimum = 0;
    double min = dev_points[0][depth];
    for (i = 1; i < 8; i++)
    {
		if (dev_points[i][depth] < min)
		{
			minimum = i;
			min = dev_points[i][depth];
		}
    }
    for (i = 0; i < 4; i++)
    {
		int j = (i+1)%4;
		int k = i+4;
		int l = 4+j;
		if (i != minimum && j != minimum)
		{
			painter->drawLine(dev_points[i][0], dev_points[i][view], dev_points[j][0], dev_points[j][view]);
		}
		if (i != minimum && k != minimum)
		{
			painter->drawLine(dev_points[i][0], dev_points[i][view], dev_points[k][0], dev_points[k][view]);
		}
		if (k != minimum && l != minimum)
		{
			painter->drawLine(dev_points[k][0], dev_points[k][view], dev_points[l][0], dev_points[l][view]);
		}
    }
    delete pen;
}


void focusContextWidget::paintFocalPoint(QPainter *painter)
{
    if (!focalPointSet)
    {
		getFocalPointPos();
		focalPointSet = true;
    }
    
    QPen *pen = new QPen(QColor("red"), 1);
    painter->setPen(*pen);
    
    if (viewDirection == 0)
    {
		painter->drawLine(dev_focalPoint[0]-2, dev_focalPoint[1], dev_focalPoint[0]+2, dev_focalPoint[1]);
		painter->drawLine(dev_focalPoint[0], dev_focalPoint[1]-2, dev_focalPoint[0], dev_focalPoint[1]+2);
    }
    else
    {
		painter->drawLine(dev_focalPoint[0]-2, dev_focalPoint[2], dev_focalPoint[0]+2, dev_focalPoint[2]);
		painter->drawLine(dev_focalPoint[0], dev_focalPoint[2]-2, dev_focalPoint[0], dev_focalPoint[2]+2);
    }
    delete pen;
}


void focusContextWidget::paintCircle(QPainter *painter)
{
	double factor = getDevFactor();
	
	QPen *pen = new QPen(QColor("red"), 1);
    painter->setPen(*pen);

	if (viewDirection == 0)
    {
		painter->drawEllipse(dev_focalPoint[0]-radius*factor, dev_focalPoint[1]-radius*factor, 2*radius*factor, 2*radius*factor);
    }
    else
    {
		painter->drawEllipse(dev_focalPoint[0]-radius*factor, dev_focalPoint[2]-radius*factor, 2*radius*factor, 2*radius*factor);
    }
    delete pen;
}

    
void focusContextWidget::rotatePoint(float *point, float *rotation)
{
	float new_point [3];

	new_point[1] = point[1]*cos((double)rotation[0]*PI/180) - point[2]*sin((double)rotation[0]*PI/180);
	new_point[2] = point[1]*sin((double)rotation[0]*PI/180) + point[2]*cos((double)rotation[0]*PI/180);	
	point[1] = new_point[1];
	point[2] = new_point[2];

	new_point[0] = point[2]*sin((double)rotation[1]*PI/180) + point[0]*cos((double)rotation[1]*PI/180);
	new_point[2] = point[2]*cos((double)rotation[1]*PI/180) - point[0]*sin((double)rotation[1]*PI/180);
	point[0] = new_point[0];
	point[2] = new_point[2];

	new_point[0] = point[0]*cos((double)rotation[2]*PI/180) - point[1]*sin((double)rotation[2]*PI/180);
	new_point[1] = point[0]*sin((double)rotation[2]*PI/180) + point[1]*cos((double)rotation[2]*PI/180);
	point[0] = new_point[0];
	point[1] = new_point[1];
}



void focusContextWidget::getPoints()
{
    float help [3];
	int i;

    help[0] = (float)dim[0]/2;
    help[1] = (float)dim[1]/2;
    help[2] = (float)dim[2]/2;
    for (i = 0; i < 8; i++)
    {
		points[i][0] = help[0];
		points[i][1] = help[1];
		if (i < 4)
			points[i][2] = help[2];
		else
			points[i][2] = -help[2];
    }
    points[0][0] = -points[0][0];
    points[3][0] = -points[3][0];
    points[4][0] = -points[4][0];
    points[7][0] = -points[7][0];
    
    points[2][1] = -points[2][1];
    points[3][1] = -points[3][1];
    points[6][1] = -points[6][1];
    points[7][1] = -points[7][1];
    

    for (i = 0; i < 8; i++)
	{
		rotatePoint(points[i], orientation);
    }
}


void focusContextWidget::getFocalPointPos()
{    
    // get Device Coordinates
    
    double factor = getDevFactor();
    int x = this->width();
    int y = this->height();
    dev_focalPoint[0] = (int)(focalPoint[0] * factor) + (int)(x/2);
    if (viewDirection == 0)
    {
		dev_focalPoint[1] = (int)(-focalPoint[1] * factor) + (int)(y/2);
		dev_focalPoint[2] = (int)(focalPoint[2] * factor) + (int)(y/2);
    }
    else
    {
		dev_focalPoint[1] = (int)(-focalPoint[1] * factor) + (int)(y/2);
		dev_focalPoint[2] = (int)(focalPoint[2] * factor) + (int)(y/2);
    }
}


float* focusContextWidget::getFocalPoint()
{
    float *help = new float [3];
	float *inv_rot = new float [3];
	inv_rot[0] = -orientation[0];
	inv_rot[1] = -orientation[1];
	inv_rot[2] = -orientation[2];
    int x = this->width();
    int y = this->height();
    
    double factor = getDevFactor();
    
    help[0] = (dev_focalPoint[0] - (x/2))/factor;
    help[1] = (dev_focalPoint[1] - (y/2))/factor;
    help[2] = (dev_focalPoint[2] - (y/2))/factor;
    help[1] = help[1]*(-1);
	rotatePoint(help, inv_rot);
    
    return help;
}


float focusContextWidget::getRadius()
{
	return (float)radius;
}


void focusContextWidget::mousePressEvent(QMouseEvent *event)
{
    dev_focalPoint[0] = event->x();
    if (viewDirection == 0)
    {
		dev_focalPoint[1] = event->y();
    }
    else
    {
		dev_focalPoint[2] = event->y();
    }
    update();
    changeFocalPoint(dev_focalPoint);
}


void focusContextWidget::updateFocalPoint(int *point)
{
    dev_focalPoint = point;
    update();
}


void focusContextWidget::setRadius(int rad)
{
	radius = (int)(((double)rad/1000)*((double)dim[0]/2));
	update();
}


void focusContextWidget::paintEvent(QPaintEvent *event)
{
    QPainter *painter = new QPainter(this);
    paintCube(painter);
    paintFocalPoint(painter);
	paintCircle(painter);
    delete painter;
}
