#ifndef FOCUSCONTEXTWIDGET_h
#define FOCUSCONTEXTWIDGET_h

#include <qwidget.h>
#include <qevent.h>
#include <qpainter.h>


//! Class focusContextWidget
/*!
    Diese Klasse ist abgeleitet von QWidget. Sie definiert die Abl&auml;ufe f&uuml;r 
	das Fenster in dem der Focus+Context-Bereich dargestellt wird.
*/
class focusContextWidget : public QWidget
{
 Q_OBJECT
    
 public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen.
	*/
    focusContextWidget(QWidget * parent = 0, const char *name = 0);
	//! Destruktor
    ~focusContextWidget();
	//! Methode setView
	/*!
		Setzt die derzeitige Ansicht in der Focus+Context Bereichsauswahl.
	*/
    void setView(int *dimension, float *orient, float *focPoint, float *middle, float rad, QString direction);
    //! Methode getFocalPoint
	/*!
		Gibt den derzeitig ausgew&auml;hlten Fokuspunkt zur&uuml;ck.
	*/
	float* getFocalPoint();
	//! Methode getRadius
	/*!
		Gibt den Radius des gew&auml;hlten Bereichs zur&uuml;ck.
	*/
	float getRadius();
    
 public slots:
	//! Slot updateFocalPoint
	/*!
		Zeichnet den Fokusbereich neu mit dem &uuml;bergebenen Fokuspunkt.
	*/
    void updateFocalPoint(int *point);
	//! Slot setRadius
	/*!
		Setzt den Radius des Fokusbereich neu.
	*/
	void setRadius(int rad);
	//! Slot setCameraFocalPoint
	/*!
		Setzt den Fokuspunkt der Focus+Context Methode auf den Fokuspunkt der Kamera.
	*/
	void setCameraFocalPoint();
    
 signals:
    void changeFocalPoint(int *point);
    
 protected:
    void mousePressEvent(QMouseEvent *event);
       
 private:
    void getPoints();
    void paintCube(QPainter *painter);
    void paintFocalPoint(QPainter *painter);
	void paintCircle(QPainter *painter);
    void paintEvent(QPaintEvent *event);
    void getFocalPointPos();
    double getDevFactor();
	void rotatePoint(float *point, float *rotation);
	    
    int viewDirection;
    float points [8][3];
    int *dim;
    float *orientation;
    float *focalPoint;
	float *camFocPoint;
    int *dev_focalPoint;
    bool focalPointSet;
	int radius;
};

#endif
