
#include "GradientFunctionViewer.h"



gradientFunctionViewer::gradientFunctionViewer(QWidget *parent, const char *name) : QWidget(parent, name)
{
	this->setBackgroundColor(QColor(245, 235, 235));
	gradientFunction[0].gradient = 0;
	gradientFunction[0].pos = 0;
	gradientFunction[1].gradient = 1;
	gradientFunction[1].pos = 1;
	points = 2;
	mousePressed = false;
	movingX = true;
	changed = false;
}


gradientFunctionViewer::~gradientFunctionViewer()
{
}


bool gradientFunctionViewer::getChanged()
{
	return changed;
}


void gradientFunctionViewer::resetChanged()
{
	changed = false;
}


void gradientFunctionViewer::resetGradientFunction()
{
	gradientFunction[0].gradient = 0;
	gradientFunction[0].pos = 0;
	gradientFunction[1].gradient = 1;
	gradientFunction[1].pos = 1;
	points = 2;
	mousePressed = false;
	movingX = true;
	update();
}


void gradientFunctionViewer::setGradientFunction(vtkPiecewiseFunction *piecewiseFunction, float range)
{
	int length = piecewiseFunction->GetSize();
	if (length < 20)
	{
		float *data = piecewiseFunction->GetDataPointer();
		for (int i = 0; i < (length-1); i++)
		{
			gradientFunction[i].pos = data[2*i]/(range/4);
			gradientFunction[i].gradient = data[2*i+1];
		}
		points = length-1;
		mousePressed = false;
		movingX = true;
		changed = true;
		update();
	}
}


void gradientFunctionViewer::getGradientFunction(vtkPiecewiseFunction *piecewiseFunction, float range)
{
	for (int i = 0; i < points; i++)
	{
		piecewiseFunction->AddPoint(gradientFunction[i].pos*range/4, gradientFunction[i].gradient);
	}
	piecewiseFunction->AddPoint(range, 1);
}


void gradientFunctionViewer::deletePoint(int index)
{
	for (int i = index; i < points - 1; i++)
	{
		gradientFunction[i] = gradientFunction[i+1];
	}
	points = points - 1;
}


void gradientFunctionViewer::insertPoint(float x, float y)
{
	for (int i = 0; i < points; i++)
	{
		if (gradientFunction[i].pos > x)
		{
			for (int j = points; j > i; j--)
			{
				gradientFunction[j].pos = gradientFunction[j-1].pos;
				gradientFunction[j].gradient = gradientFunction[j-1].gradient;
			}
			gradientFunction[i].pos = x;
			gradientFunction[i].gradient = y;
			i = points;
		}
	}
	points = points + 1;
}


int gradientFunctionViewer::testIntersection(int x, int y, int index)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	for (int i = 0; i < points; i++)
	{
		if (i != index)
		{
			int xPos = (int)(gradientFunction[i].pos * factorX) + 3;
			int yPos = height - (int)(gradientFunction[i].gradient * factorY) - 3;
			if (x > xPos-3 && x < xPos+3 && y > yPos-3 && y < yPos+3)
				return i;
		}
	}
	return -1;
}


void gradientFunctionViewer::paintCubes(QPainter *paint)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	for (int i = 0; i < points; i++)
	{
		int xPos = (int)(gradientFunction[i].pos * factorX) + 3;
		int yPos = height - (int)(gradientFunction[i].gradient * factorY) - 3;
		paint->drawLine(xPos-3, yPos-3, xPos+3, yPos-3);
		paint->drawLine(xPos+3, yPos-3, xPos+3, yPos+3);
		paint->drawLine(xPos+3, yPos+3, xPos-3, yPos+3);
		paint->drawLine(xPos-3, yPos+3, xPos-3, yPos-3);
	}
}


void gradientFunctionViewer::paintLines(QPainter *paint)
{
	int width = this->width();
	int height = this->height();
	double factorX = width - 6;
	double factorY = height - 6;
	for (int i = 1; i < points; i++)
	{
		int xPos1 = (int)(gradientFunction[i-1].pos * factorX) + 3;
		int yPos1 = height - (int)(gradientFunction[i-1].gradient * factorY) - 3;
		int xPos2 = (int)(gradientFunction[i].pos * factorX) + 3;
		int yPos2 = height - (int)(gradientFunction[i].gradient * factorY) - 3;
		paint->drawLine(xPos1, yPos1, xPos2, yPos2);
	}
}


void gradientFunctionViewer::mousePressEvent(QMouseEvent *event)
{
	mousePressed = true;
	intersectionIndex = testIntersection(event->x(), event->y(), -1);
	if (intersectionIndex == 0 || intersectionIndex == points-1)
		movingX = false;
}


void gradientFunctionViewer::mouseReleaseEvent(QMouseEvent *event)
{
	mousePressed = false;
	movingX = true;
}


void gradientFunctionViewer::mouseDoubleClickEvent(QMouseEvent *event)
{
	if (points < 20)
	{
		int width = this->width();
		int height = this->height();
		int x = event->x();
		int y = height - event->y();
		if (x > 3 && x < width-4 && y > 3 && y < height-4)
		{
			double factorX = width - 6;
			double factorY = height - 6;
			int intersect = testIntersection(x, y, -1);
			if (intersect < 0)
			{
				insertPoint((float)((x-3)/factorX), (float)((y-3)/factorY));
				changed = true;
				update();
			}
		}
	}
}


void gradientFunctionViewer::mouseMoveEvent(QMouseEvent *event)
{
	if (mousePressed == true && intersectionIndex >= 0)
	{
		int width = this->width();
		int height = this->height();
		double factorX = width - 6;
		double factorY = height - 6;
		int x = event->x() - 3;
		int y = height - event->y() - 3;
		float xPos = (float)((x)/factorX);
		float yPos = (float)((y)/factorY);
		if (yPos < 0)
			yPos = 0;
		if (yPos > 1)
			yPos = 1;
		if (intersectionIndex != 0 && intersectionIndex != points-1)
		{
			if (x > (int)(gradientFunction[intersectionIndex-1].pos * factorX)+1 && x < (int)(gradientFunction[intersectionIndex+1].pos * factorX)-1)
				movingX = true;
			int intersect = testIntersection(event->x(), event->y(), intersectionIndex);
			if (intersect >= 0)
			{
				deletePoint(intersectionIndex);
				update();
				movingX = false;
				intersectionIndex = -1;
			}
		}
		if (movingX == false && intersectionIndex >= 0)
		{
			gradientFunction[intersectionIndex].gradient = yPos;
			changed = true;
			update();
		}
		else if (movingX == true && intersectionIndex >= 0)
		{
			if (x <= (int)(gradientFunction[intersectionIndex-1].pos * factorX)+1 || x >= (int)(gradientFunction[intersectionIndex+1].pos * factorX)-1)
				movingX = false;
			else
			{
				gradientFunction[intersectionIndex].pos = xPos;
				gradientFunction[intersectionIndex].gradient = yPos;
				changed = true;
				update();
			}
		}
	}
}


void gradientFunctionViewer::paintEvent(QPaintEvent *event)
{
	QPainter *paint = new QPainter(this);
	paintCubes(paint);
	paintLines(paint);
	delete paint;
}