#ifndef GRADIENTFUNCTIONVIEWER_H
#define GRADIENTFUNCTIONVIEWER_H


#include <qwidget.h>
#include <qevent.h>
#include <qpainter.h>

#include "vtkPiecewiseFunction.h"
#include "GradientFunctionStruct.h"


//! Class gradientFunctionViewer
/*!
    Die Klasse ist abgeleitet von QWidget. Sie steuert die Abl&auml;fe im Widget, welches
	die Opacity Transfer Function anzeigt.
*/
class gradientFunctionViewer : public QWidget
{

public:
	//! Konstruktor
	/*!
		Definiert das Parent Widget und gibt dem Widget einen Namen.
	*/
	gradientFunctionViewer(QWidget *parent, const char *name);
	//! Destruktor
	~gradientFunctionViewer();
	//! Methode resetGradientFunction
	/*!
		Setzt die Struktur der Gradienten Funktionen zur&uuml;ck.
	*/
	void resetGradientFunction();
	//! Methode setGradientFunction
	/*!
		Setzt den Gradient Viewer auf die &uuml;bergebenen Werte.
	*/
	void setGradientFunction(vtkPiecewiseFunction *piecewiseFunction, float range);
	//! Methode getGradientFunction
	/*!
		Gibt die Einstellungen des Gradient Viewer zur&uuml;ck. range Beschreibt den
		ganzen Wertebereich der Daten.
	*/
	void getGradientFunction(vtkPiecewiseFunction *piecewiseFunction, float range);
	//! Methode getChanged
	/*!
		Gibt true zur&uuml;ck, wenn sich etwas im Gradient Viewer ge&auml;ndert hat.
	*/
	bool getChanged();
	//! Methode resetChanged
	/*!
		Setzt boolsche Variable, welche &Auml;nderungen angibt, auf false.
	*/
	void resetChanged();

protected:
    void paintEvent(QPaintEvent *event);
	void mousePressEvent(QMouseEvent *event);
	void mouseReleaseEvent(QMouseEvent *event);
	void mouseDoubleClickEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);

private:
	void paintCubes(QPainter *paint);
	void paintLines(QPainter *paint);
	void insertPoint(float x, float y);
	void deletePoint(int index);
	int testIntersection(int x, int y, int index);

	gradientFunc gradientFunction [20];
	int points;
	bool mousePressed;
	bool movingX;
	int intersectionIndex;
	bool changed;

};

#endif