#include <qslider.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <math.h>


#include "MLAnim_newPointImpl.h"
#include "MLDirectionEditorImpl.h"


#define PI 3.14159265


MLAnim_newPointImpl::MLAnim_newPointImpl(QWidget *parent, const char *name, MLAnim_struct newPoint, MLAnim_struct oldPoint) : MLAnim_newPoint(parent, name)
{
	anim = newPoint;
	animRef = oldPoint;
	setValues();
	exitStatus = false;
}


MLAnim_newPointImpl::~MLAnim_newPointImpl()
{
}


MLAnim_struct MLAnim_newPointImpl::getMLAnimStruct()
{
	return anim;
}


bool MLAnim_newPointImpl::getExitStatus()
{
	return exitStatus;
}


bool MLAnim_newPointImpl::getValues()
{
	anim.angle_diff = angleDiffSlider->value();
	anim.transfer = (float)transferFSlider->value()/100;
	
	// Slider
	anim.xRot = xSlider->value();
	anim.yRot = ySlider->value();
	anim.zRot = zSlider->value();
	anim.zoom = (float)zoomSlider->value()/100;

	anim.xClockwise = xRotClockwise->isChecked();
	anim.yClockwise = yRotClockwise->isChecked();
	anim.zClockwise = zRotClockwise->isChecked();

	anim.samples = samplesSlider->value();

	return true;
}


void MLAnim_newPointImpl::setValues()
{
	// Magic Lamp Settings
	angleDiffSlider->setValue(anim.angle_diff);
	transferFSlider->setValue((int)(anim.transfer*100));
	angleDiffLabel->setText(QString::number(anim.angle_diff));
	transferFLabel->setText(QString::number(anim.transfer));

	// Rotation
	xSlider->setValue(anim.xRot);
	ySlider->setValue(anim.yRot);
	zSlider->setValue(anim.zRot);
	zoomSlider->setValue((int)(anim.zoom*100));

	xRotLabel->setText(QString::number(anim.xRot));
	yRotLabel->setText(QString::number(anim.yRot));
	zRotLabel->setText(QString::number(anim.zRot));
	zoomLabel->setText(QString::number(anim.zoom));

	xRotClockwise->setChecked(anim.xClockwise);
	yRotClockwise->setChecked(anim.yClockwise);
	zRotClockwise->setChecked(anim.zClockwise);
	
	int diffX, diffY, diffZ;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;

	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;

	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;

	xRotLabel_diff->setText(QString::number(diffX));
	yRotLabel_diff->setText(QString::number(diffY));
	zRotLabel_diff->setText(QString::number(diffZ));

	samplesSlider->setValue(anim.samples);
	samplesLabel->setText(QString::number(anim.samples));
}


void MLAnim_newPointImpl::okayButtonClicked()
{
	if (getValues())
	{
		exitStatus = true;
		close();
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "One or more Values are not valid float numbers.");
}


void MLAnim_newPointImpl::setMLButtonClicked()
{
    MLDirectionEditorImpl *mlDirEditor = new MLDirectionEditorImpl(this, "MLDirEditor", anim.sideDir, anim.uprightDir);
	mlDirEditor->exec();
	if (mlDirEditor->getExitStatus())
	{
		float upright = mlDirEditor->getUprightDirection();
		float side = mlDirEditor->getSideDirection();
		anim.direction[2] = cos(upright*PI/180) * cos(side*PI/180);
		anim.direction[0] = (-1)*sin(upright*PI/180) * cos(side*PI/180);
		anim.direction[1] = (-1)*sin(side*PI/180);
		float length = sqrt(anim.direction[0]*anim.direction[0] + anim.direction[1]*anim.direction[1]+ anim.direction[2]*anim.direction[2]);
		anim.direction[0] /= length;
		anim.direction[1] /= length;
		anim.direction[2] /= length;
		anim.uprightDir = upright;
		anim.sideDir = side;
	}
	delete mlDirEditor;
}


void MLAnim_newPointImpl::angleDiffSliderChanged(int value)
{
	angleDiffLabel->setText(QString::number(value));
	anim.angle_diff = value;
}


void MLAnim_newPointImpl::transferFSliderChanged(int value)
{
	transferFLabel->setText(QString::number((float)value/100));
	anim.transfer = (float)value/100;
}


void MLAnim_newPointImpl::xRotSliderChanged(int value)
{
	xRotLabel->setText(QString::number(value));
	anim.xRot = value;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void MLAnim_newPointImpl::yRotSliderChanged(int value)
{
	yRotLabel->setText(QString::number(value));
	anim.yRot = value;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void MLAnim_newPointImpl::zRotSliderChanged(int value)
{
	zRotLabel->setText(QString::number(value));
	anim.zRot = value;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}


void MLAnim_newPointImpl::zoomSliderChanged(int value)
{
	zoomLabel->setText(QString::number((float)value/100));
	anim.zoom = (float)value/100;
}


void MLAnim_newPointImpl::samplePointsSliderChanged(int value)
{
	samplesLabel->setText(QString::number(value));
	anim.samples = value;
}


void MLAnim_newPointImpl::xRotClockwiseClicked(bool status)
{
	anim.xClockwise = status;
	int diffX;
	if (anim.xRot-animRef.xRot >= 0)
	{
		if (anim.xClockwise)
			diffX = anim.xRot-animRef.xRot;
		else
			diffX = 360 - (anim.xRot-animRef.xRot);
	}
	else
	{
		if (anim.xClockwise)
			diffX = 360 + (anim.xRot-animRef.xRot);
		else
			diffX = -(anim.xRot-animRef.xRot);
	}
	if (diffX == 360)
		diffX = 0;
	xRotLabel_diff->setText(QString::number(diffX));
}


void MLAnim_newPointImpl::yRotClockwiseClicked(bool status)
{
	anim.yClockwise = status;
	int diffY;
	if (anim.yRot-animRef.yRot >= 0)
	{
		if (anim.yClockwise)
			diffY = anim.yRot-animRef.yRot;
		else
			diffY = 360 - (anim.yRot-animRef.yRot);
	}
	else
	{
		if (anim.yClockwise)
			diffY = 360 + (anim.yRot-animRef.yRot);
		else
			diffY = -(anim.yRot-animRef.yRot);
	}
	if (diffY == 360)
		diffY = 0;
	yRotLabel_diff->setText(QString::number(diffY));
}


void MLAnim_newPointImpl::zRotClockwiseClicked(bool status)
{
	anim.zClockwise = status;
	int diffZ;
	if (anim.zRot-animRef.zRot >= 0)
	{
		if (anim.zClockwise)
			diffZ = anim.zRot-animRef.zRot;
		else
			diffZ = 360 - (anim.zRot-animRef.zRot);
	}
	else
	{
		if (anim.zClockwise)
			diffZ = 360 + (anim.zRot-animRef.zRot);
		else
			diffZ = -(anim.zRot-animRef.zRot);
	}
	if (diffZ == 360)
		diffZ = 0;
	zRotLabel_diff->setText(QString::number(diffZ));
}