#include <qlistbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>

#include "MLAnimationEditorImpl.h"
#include "MLAnim_startPointImpl.h"
#include "MLAnim_newPointImpl.h"



MLAnimationEditorImpl::MLAnimationEditorImpl(QWidget* parent, const char* name, MLAnim_struct startPoint) : MLAnimationEditor(parent, name)
{
	exitStatus = false;
	anim = new MLAnim_struct [30];
	anim[0] = startPoint;
	number = 1;
	frames = 0;
	frameCounter->setText("0");
}


MLAnimationEditorImpl::~MLAnimationEditorImpl()
{
}


bool MLAnimationEditorImpl::getExitStatus()
{
	return exitStatus;
}


MLAnim_struct *MLAnimationEditorImpl::getMLAnimStruct()
{
	return anim;
}


int MLAnimationEditorImpl::getNumbers()
{
	return number;
}


void MLAnimationEditorImpl::deletePointButtonClicked()
{
	int pos = pointBox->currentItem();
	if (pos >= 0)
	{
		for (int i = pos+1; i < number-1; i++)
		{
			anim[i] = anim[i+1];
		}
		number = number-1;
		pointBox->removeItem(pos);
		if (pos+1 < number)
			pointBox->setCurrentItem(pos);
		else
			pointBox->setCurrentItem(pos-1);
		frames = 0;
		for (i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
	}
}


void MLAnimationEditorImpl::editPointButtonClicked()
{
	int pos = pointBox->currentItem()+1;
	MLAnim_newPointImpl *editPoint = new MLAnim_newPointImpl(this, "Edit Point", anim[pos], anim[pos-1]);
	editPoint->exec();
	if (editPoint->getExitStatus())
	{
		anim[pos] = editPoint->getMLAnimStruct();
		frames = 0;
		for (int i = 1; i < number; i++)
			frames += anim[i].samples; 
		frameCounter->setText(QString::number(frames));
		QString frameString = "[" + QString::number(anim[pos].samples) + " frames]";
		pointBox->changeItem(frameString, pos-1);
	}
	delete editPoint;
}


void MLAnimationEditorImpl::newPointButtonClicked()
{
	if (number <= 30)
	{
		MLAnim_newPointImpl *newPoint = new MLAnim_newPointImpl(this, "Edit Point", anim[number-1], anim[number-1]);
		newPoint->exec();
		if (newPoint->getExitStatus())
		{
			anim[number] = newPoint->getMLAnimStruct();
			number = number+1;
			QString frameString = "[" + QString::number(anim[number-1].samples) + " frames]";
			pointBox->insertItem(frameString, -1);
			frames = 0;
			for (int i = 1; i < number; i++)
				frames += anim[i].samples; 
			frameCounter->setText(QString::number(frames));
		}
		delete newPoint;
	}
	else
		QMessageBox::warning(this, "Depth of Field - Renderer", "Maximum Point Count is reached");
}


void MLAnimationEditorImpl::startPointButtonClicked()
{
	MLAnim_startPointImpl *startPoint = new MLAnim_startPointImpl(this, "Edit Point", anim[0]);
	startPoint->exec();
	if (startPoint->getExitStatus())
	{
		anim[0] = startPoint->getMLAnimStruct();
	}
	delete startPoint;
}



void MLAnimationEditorImpl::startAnimationButtonClicked()
{
	if (number > 1)
		exitStatus = true;
	close();
}