
#include <iostream.h>
#include <qcolor.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qslider.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qcolordialog.h>
#include <qfiledialog.h>

#include "transferFunctionEditorImpl.h"
#include "transferFunctionViewer.h"
#include "tfWriter.h"
#include "tfReader.h"


transferFunctionEditorImpl::transferFunctionEditorImpl (QWidget* parent, const char* name, unsigned int histo [256]) : transferFunctionEditor (parent, name)
{
    changes = false;
    exitStatus = false;
	saveStatus = false;
    beginChange = false;
    endChange = false;
    nameChange = false;
    deleteChange = false;
    
    // set Histogram in the Transfer Viewer
    viewer->setHistogram (histo);
	transFunc = new transferFunction ();
    
    // reset all values and set the Controlelements and Labels to "not enable"
    setEnableElements (false);
    saveButton->setEnabled (false);
}


transferFunctionEditorImpl::transferFunctionEditorImpl (QWidget* parent, const char* name, unsigned int histo [256], transferFunction transFunction) : transferFunctionEditor (parent, name)
{
    changes = false;
    exitStatus = false;
	saveStatus = false;
    beginChange = false;
    endChange = false;
    nameChange = false;
    deleteChange = false;
    
    // set Values of the Transfer-Viewer Widget in the Editor
    transFunc = new transferFunction(transFunction);
    viewer->setHistogram (histo);
    viewer->setTransferFunction (transFunc);
    saveButton->setEnabled (false);
    
    //
    transfer_function tf = transFunc->tf;
    if (tf.length == 0)
    {
		setEnableElements (false);
    }
    else
    {
		setEnableElements (true);
		setEditorFields ();
		viewer->setCurrent (0);
    }
}


transferFunctionEditorImpl::~transferFunctionEditorImpl()
{   
    delete viewer;
    delete transFunc;
}


bool transferFunctionEditorImpl::getExitStatus()
{
    return exitStatus;
}


bool transferFunctionEditorImpl::getSaveStatus()
{
	return saveStatus;
}


bool transferFunctionEditorImpl::getChanges()
{
    return changes;
}


transferFunction* transferFunctionEditorImpl::getTransferFunction ()
{
    return transFunc;
}


void transferFunctionEditorImpl::setEditorFields ()
{
    transfer_function tf = transFunc->tf;
    functionSelector->clear ();
    for (int i = 0; i < tf.length; i++)
    {
		functionSelector->insertItem (tf.element[i].name);
    }
    functionSelector->setCurrentItem (0);
	functionSelector->setSelected (0, true);
    transfer_element element = tf.element[0];
    setCurrentValues (element);
}


void transferFunctionEditorImpl::setCurrentValues (transfer_element element)
{
    nameChange = true;
    nameEdit->setText (element.name);
    nameChange = false;
    int opacity = (int)(element.opacity * 100);
    opacitySlider->setValue (opacity);
    int begin = 0;
    //if (element.begin+element.length > 0)
    {
		begin = (int)(beginSlider->maxValue()/(element.begin+element.length) *element.begin);
    }
    beginSlider->setValue (begin);
    int end = (int)(endSlider->maxValue()/(256-element.begin) * element.length);
    endSlider->setValue (end);
    slope_leftSlider->setValue (element.angle_left);
    slope_rightSlider->setValue (element.angle_right);
    changes = false;
	setEnableElements (true);
}


void transferFunctionEditorImpl::setEnableElements (bool enable)
{
    // Steuerelemente
    nameEdit->setEnabled (enable);
    changeColorButton->setEnabled (enable);
    opacitySlider->setEnabled (enable);
    beginSlider->setEnabled (enable);
    endSlider->setEnabled (enable);
    slope_leftSlider->setEnabled (enable);
    slope_rightSlider->setEnabled (enable);

	// Slider auf 0 setzen
	if (enable == false)
	{
		opacitySlider->setValue (0);
		beginSlider->setValue (0);
		endSlider->setValue (0);
		slope_leftSlider->setValue (0);
		slope_rightSlider->setValue (0);
	}
    
    // Labels    
    Name->setEnabled (enable);
    Color->setEnabled (enable);
    Opacity->setEnabled (enable);
    Begin->setEnabled (enable);
    End->setEnabled (enable);
    Slope_Left->setEnabled (enable);
    Slope_Right->setEnabled (enable);
    
    // Buttons
    deleteButton->setEnabled (enable);
}


void transferFunctionEditorImpl::functionSelectorPressed (QListBoxItem * item)
{
    if (!deleteChange && transFunc->tf.length != 0 && functionSelector->index (item) >= 0)
    {
		int index = functionSelector->index (item); 
		transfer_function tf = transFunc->tf;
		transfer_element element = tf.element[index];
		if (!nameChange)
		{
			if (!nameChangedExit ())
			{
				viewer->setCurrent (index);
				setCurrentValues (element);
			}
		}
		else
		{
			viewer->setCurrent (index);
			setCurrentValues (element);
		}
    }
}


void transferFunctionEditorImpl::nameChanged (const QString &newName)
{
    
    if (!nameChange)
    {
		int index = functionSelector->currentItem ();
		transfer_function tf = transFunc->tf;
	
		if (index >= 0)
		{
			if (tf.element[index].name.compare(newName) != 0)
			{
				tf.element[index].name = newName;
				changes = true;
				deleteChange = true;
				functionSelector->changeItem (newName, index);
				deleteChange = false;
				saveButton->setEnabled (true);
			}
		}
    }
}


bool transferFunctionEditorImpl::nameChangedExit ()
{
    transfer_function tf = transFunc->tf;
    //cout << "hier bin ich \n";
    int i, index = tf.length;
    for (i = 0; i < tf.length; i++)
    {
	if (tf.element[i].name.length() == 0)
	{
	    index = i;
	    i = tf.length;
	}
    }
    if (index < tf.length)
    {		
		//nameEdit->setText(tf.element[index].name);
		//nameEdit->setFocus ();
		viewer->setCurrent (index);
		setCurrentValues (tf.element[index]);
		//functionSelector->setSelected (index, true);  // setCurrentItem
		nameChange = true;
		functionSelector->setCurrentItem (index);
		nameChange = false;
		QMessageBox::warning( this, "Depth of Field - Renderer", "Name could not be empty." );
		return true;
		//nameEdit->setCursorPosition (0);
    }
    return false;
}


void transferFunctionEditorImpl::opacityChanged (int value)
{
    int index = functionSelector->currentItem ();
    transfer_function tf = transFunc->tf;
    tf.element[index].opacity = (float)value/100;
    viewer->update ();
    saveButton->setEnabled (true);
}


void transferFunctionEditorImpl::beginChanged (int value)
{
    if (!endChange)
    {
		beginChange = true;
		int index = functionSelector->currentItem ();
		transfer_function tf = transFunc->tf;
		float end = tf.element[index].begin + tf.element[index].length;
		tf.element[index].begin = end*(float)value/1000;
		tf.element[index].length = end - tf.element[index].begin;
		int end_value = (int)(endSlider->maxValue()/(256-tf.element[index].begin) * tf.element[index].length);
		endSlider->setValue (end_value);
		viewer->update ();
		beginChange = false;
		saveButton->setEnabled (true);
    }
}


void transferFunctionEditorImpl::endChanged (int value)
{
    if (!beginChange) 
    {
		endChange = true;
		int index = functionSelector->currentItem ();
		transfer_function tf = transFunc->tf;
		float begin = 256 - tf.element[index].begin;
		tf.element[index].length = begin*(float)value/1000;
		int begin_value = (int)(beginSlider->maxValue()/(tf.element[index].begin+tf.element[index].length) * tf.element[index].begin);
		beginSlider->setValue (begin_value);
		viewer->update ();
		endChange = false;
		saveButton->setEnabled (true);
    }
}


void transferFunctionEditorImpl::slope_leftChanged (int value)
{
    int index = functionSelector->currentItem ();
    transfer_function tf = transFunc->tf;
    tf.element[index].angle_left = value;
    viewer->update ();
    saveButton->setEnabled (true);
}


void transferFunctionEditorImpl::slope_rightChanged (int value)
{
    int index = functionSelector->currentItem ();
    transfer_function tf = transFunc->tf;
    tf.element[index].angle_right = value;
    viewer->update ();
    saveButton->setEnabled (true);
}


void transferFunctionEditorImpl::colorButtonPressed ()
{
    int index = functionSelector->currentItem ();
    bool flag = false;
    transfer_function tf = transFunc->tf;
    QColor color = QColorDialog::getRgba (tf.element[index].color.rgb (), &flag, this, "Depth of Field - Color Chooser");
    if (flag == true)
    {
		tf.element[index].color = color;
		viewer->update ();
		saveButton->setEnabled (true);
    }
}


void transferFunctionEditorImpl::newButtonPressed ()
{
    transfer_element element;
    transfer_function tf = transFunc->tf;
    element.name = "unnamed" + QString::number (tf.length);
    element.opacity = 0;
    element.begin = 0;
    element.length = 0;
    element.angle_left = 0;
    element.angle_right = 0;
    element.color = QColor (255, 255, 255);
    transFunc->addElement (element);
    functionSelector->insertItem (element.name);
    functionSelector->setSelected (tf.length, true);
	viewer->setCurrent (tf.length);
    setEnableElements (true);
    setCurrentValues (element);
    saveButton->setEnabled (true);
}


void transferFunctionEditorImpl::deleteButtonPressed ()
{
    int new_index;
	int index = functionSelector->currentItem ();
    if (index == (int)functionSelector->count () - 1)
    {
		new_index = index - 1;
    }
    else
    {
		new_index = index;
    }    
    deleteChange = true;
    functionSelector->setSelected (index, false);
    functionSelector->removeItem (index);   
    transFunc->removeElement (index);
    deleteChange = false;
    saveButton->setEnabled (true);
    if (new_index >= 0)
    {
		functionSelector->setSelected (new_index, true);
		transfer_function tf = transFunc->tf;
		setCurrentValues (tf.element[new_index]);
    }
    else
    {
		nameEdit->setText(QString (""));
		viewer->update ();
		setEnableElements (false);
    }
}


void transferFunctionEditorImpl::saveButtonPressed ()
{
    QString saveFile = QFileDialog::getSaveFileName(
                    "",
                    "TransferF-F (*.tff)",
                    this,
                    "save file dialog"
                    "Choose a filename to save under" );
    if (!saveFile.isEmpty ())
    {
		if (!saveFile.endsWith(".tff", false))
			saveFile = saveFile + ".tff";
		TFWriter writer = TFWriter(saveFile);
		writer.write(transFunc);
    }
	saveStatus = true;
}


void transferFunctionEditorImpl::loadButtonPressed ()
{
    QString openFile = QFileDialog::getOpenFileName(
                    "",
                    "TransferF-F (*.tff)",
                    this,
                    "open file dialog"
                    "Choose a file to Open" );
    if (!openFile.isEmpty())
    {
		TFReader reader = TFReader (openFile);
		if (reader.read ())
		{
			transFunc = new transferFunction (*reader.getTransferFunctions());
			setEditorFields ();
			viewer->setTransferFunction (transFunc);
			viewer->setCurrent (0);
			saveButton->setEnabled (false);
		}
		else
		{
			QMessageBox::warning( this, "Depth of Field - Renderer", "Error while reading from Transfer Function File." );
		}
		//delete reader;
    }
}


void transferFunctionEditorImpl::okayButtonPressed ()
{
    exitStatus = true;
    this->reject ();
}

