
#include <iostream.h>
#include <qmessagebox.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qregexp.h>
#include <qcheckbox.h>
#include <math.h>
#include <qlabel.h>
#include <qtabwidget.h>
#include <qcolordialog.h>

#include "ViewWidgetImpl.h"
#include "TransferFunctionEditorImpl.h"
#include "Vtkbox.h"
#include "Dimdialog.h"
#include "Reader.h"
#include "TFReader.h"
#include "GOReader.h"
#include "GOWriter.h"
#include "TransferFunctionViewer.h"
#include "FocalEditorImpl.h"
#include "DOFAnimationEditorImpl.h"
#include "GradientFunctionViewer.h"
#include "FocusContextEditorImpl.h"
#include "ContextOTFunctionViewer.h"
#include "FCReader.h"
#include "FCWriter.h"
#include "FCAnimationEditorImpl.h"
#include "FCAnim_struct.h"
#include "MLDirectionEditorImpl.h"
#include "DithAnim_struct.h"
#include "MLAnim_struct.h"
#include "DithAnimationEditorImpl.h"
#include "MLAnimationEditorImpl.h"


#define PI 3.14159265


viewWidgetImpl::viewWidgetImpl(QWidget *parent, const char *name) : viewWidget (parent, name)
{
    // set Statusvariablen
	interactive = false;
    xangle = 0;
    yangle = 0;
    zangle = 0;
	xstatus = 0;
	ystatus = 0;
	zstatus = 0;
	zoom = 1;
    zoomvalue = 1;
	background[0] = 0;
	background[1] = 0;
	background[2] = 0;
	background_changed = false;
	range = 256;
	parallelViewStatus = false;
	perspectiveViewStatus = true;
	sampleDistance = 1;

	dith_distance = 1;
	dith_size = 1;
	dith_context = true;
	dith_structure = true;
	dith_second_row = true;
	dith_second_row_structure = false;

	ML_direction[0] = 0;
	ML_direction[1] = 0;
	ML_direction[2] = 1;
	ML_angle = 15;
	ML_transferF = 0;
	ML_backside = false;
	ML_sideDir = 0;
	ML_uprightDir = 0;

	status_changed = false;

	FC_changes = false;
	DOF_changes = false;
	ML_changes = false;
	Dith_changes = false;

	DOF_status = false;
	FC_status = false;
	ML_status = false;
	Dith_status = false;

	DOF_randomSP_status = false;

	for (int i = 0; i < 256; i++)
		histo[i] = 0;
    file_loaded = false;
    openEditorButton->setEnabled(false);
    renewValues();
    setEnabledWidgets(false);
}


viewWidgetImpl::~viewWidgetImpl()
{
	delete VTKview;
    delete transFunc;
}


void viewWidgetImpl::openEditor()
{
    if (file_loaded == true)
    {
		transferFunctionEditorImpl *editor = new transferFunctionEditorImpl(this, "editor", histo, *transFunc);
		editor->exec();
		if (editor->getExitStatus())
		{
			transFunc = new transferFunction(*editor->getTransferFunction());
			TFPreview->setTransferFunction(transFunc);
			TFPreview->update();
			vtkColorTransferFunction *colorFunc = colorFunction();
            vtkPiecewiseFunction *opacityFunc = opacityFunction();
            VTKview->repaintTF(colorFunc, opacityFunc);
		}
		delete editor;
    }
    else
    {
		QMessageBox::warning(this, "Depth of Field - Renderer", "Could not open Transfer Function Editor.\n No File was loaded.");
    }
}


void viewWidgetImpl::loaded(vtkImageData *imageData)
{		
	setRenderStatus("Loading File");
	setFileStatus("... Loading");

	// set Renderer and Functions Preview
	setEnabledWidgets(true);
    defaultValues();
    vtkColorTransferFunction *colorFunc = colorFunction();
    vtkPiecewiseFunction *opacityFunc = opacityFunction();
	vtkPiecewiseFunction *gradientFunc = vtkPiecewiseFunction::New();
	gradientViewer->getGradientFunction(gradientFunc, range);
	VTKview->setRenderer(imageData, colorFunc, opacityFunc);
	VTKview->rendering(gradientFunc);
    TFPreview->setHistogram(histo);
    TFPreview->setTransferFunction(transFunc);
    TFPreview->update();
	contextOTFViewer->setHistogram(histo);
    openEditorButton->setEnabled(true);

	vtkColorTransferFunction *cTF = vtkColorTransferFunction::New();
	cTF->AddRGBPoint(0, (float)FC_color_status.red()/256, (float)FC_color_status.green()/256, (float)FC_color_status.blue()/256);
	cTF->AddRGBPoint(range, (float)FC_color_status.red()/256, (float)FC_color_status.green()/256, (float)FC_color_status.blue()/256);
	vtkPiecewiseFunction *cPWF = vtkPiecewiseFunction::New();
	contextOTFViewer->getOpacityFunction(cPWF, range);
	vtkPiecewiseFunction *conOTF = vtkPiecewiseFunction::New();
	conOTF->AddPoint(30, 0.1);
	conOTF->AddPoint(255, 0.1);
	VTKview->renderingFC(cTF, cPWF, FC_transferCoefficient_status, FC_ksc_status, FC_kss_status, FC_kse_status, FC_redOffset_status, FC_redFactor_status);
	VTKview->renderingFC(FC_middle, FC_radius);

	VTKview->setDitheringParameters(1, 1, true, true, true, false);
	//VTKview->setDithering(true);

	//VTKview->setMagicLamp(true);

	VTKview->RenderImage();

    file_loaded = true;
}


void viewWidgetImpl::openFile()
{
    bool flag = false, dimensionRead = false;
    QString filename = QFileDialog::getOpenFileName(
			    QString::null, "Volume (*.dat *.raw)", this,
			    "file open", "File Open" );
    
    if (!filename.isEmpty())
    {
		//dimension = readDimension (filename);
		readDimension(filename);
	
		//  read Dimension from filename or from DimDialog, if the filename contain no Dimension
		if (dimension == NULL)
		{
			DimDialog *dim = new DimDialog(this);
			dim->exec();
			if (dim->flag)
			{
				dim->getValues (dimension);
				dimensionRead =true;
			}
			delete dim;
		}
		else
		{
			dimensionRead = true;
		}
	
		// if Dimensions could be read
		if (dimensionRead)
		{
			flag = true;
			readSpacing(filename);
			Reader *imageReader = new Reader(filename, dimension, spacing);
			range = imageReader->getRange();
			FC_middle[0] = dimension[0]/2;
			FC_middle[1] = dimension[1]/2;
			FC_middle[2] = dimension[2]/2;
			FC_radius = dimension[0]/4;
			if (imageReader->getErrorReport() == 0)
			{
				transFunc = new transferFunction();
				TFReader *tf_reader = new TFReader(tfFileName);
				if (!tf_reader->read())
				{
					QMessageBox::information(this, "Depth of Field - Renderer", "Could not load Transfer Function File.");
				}
				else 
				{
					transFunc = tf_reader->getTransferFunctions();
				}
				delete tf_reader;
				GOReader *go_reader = new GOReader(goFileName);
				if (!go_reader->read())
					gradientViewer->resetGradientFunction();
				else
				{
					gradientViewer->setGradientFunction(go_reader->getPiecewiseFunction(), range);
					gradientViewer->resetChanged();
				}
				delete go_reader;
				FCReader *fc_reader = new FCReader(fcFileName);
				if (!fc_reader->read())
				{
					contextOTFViewer->resetOpacityFunction();
					setFCValues(QColor(255, 255, 255), 2, 0.05, 0.95, 0.05, 1, 1);
				}
				else
				{
					float ksc, kss, kse, transC, redOffset, redFactor;
					QColor color;
					vtkPiecewiseFunction *pwFunc = vtkPiecewiseFunction::New();
					fc_reader->getFCSettings(pwFunc, &color, &transC, &ksc, &kss, &kse, &redOffset, &redFactor);
					contextOTFViewer->setOpacityFunction(pwFunc, range);
					setFCValues(color, transC, ksc, kss, kse, redOffset, redFactor);
				}
				delete fc_reader;

				factor = imageReader->getFactor();
				imageReader->getHistogram(histo);
				loaded(imageReader->getData());
				setFileStatus(filename.section('/', -1) + " loaded");
			}
			else
			{
				if (imageReader->getErrorReport() == 1)
				{
					QMessageBox::warning(this, "Depth of Field - Renderer", "Could not open file.");
				}
				else
				{
					QMessageBox::warning(this, "Depth of Field - Renderer", "Wrong File Dimensions.");
				}
			}
			delete imageReader;
		}
    }
    else         // if no File selected
    {
		flag = true;
    }
    
    // if no Data loaded while no Dimensions where found
    if (flag == false)
    {
		QMessageBox::information( this, "Depth of Field", "Can not read file" );
    }
}


void viewWidgetImpl::setFocusButtonPressed()
{
    int new_dim [3];
    new_dim [0] = (int)(dimension [0] * spacing [0]);
    new_dim [1] = (int)(dimension [1] * spacing [1]);
    new_dim [2] = (int)(dimension [2] * spacing [2]);
    focalEditorImpl *focEditor = new focalEditorImpl(this, "Renderer - Set Focal Point", new_dim, VTKview->getOrientation(), VTKview->getFocalPoint());
    focEditor->exec();
    if (focEditor->okay)
    {
		VTKview->setFocalPoint(focEditor->getFocalPoint());
    }
	delete focEditor;
}


void viewWidgetImpl::readDimension(QString filename)
{
    bool ok;

    QRegExp rxp1 = QRegExp("[0-9]+x[0-9]+x[0-9]", TRUE, FALSE);
    QRegExp rxp2 = QRegExp("[^x0-9]", TRUE, FALSE);
    QRegExp rxp3 = QRegExp("_*[0-9]+x[0-9]+x[0-9]", TRUE, FALSE);
    
    QString dim = filename.section(rxp1, 1, 1, QString::SectionIncludeLeadingSep);    
    tfFileName = filename.section(rxp3, 0, 0) + ".TFF";
	goFileName = filename.section(rxp3, 0, 0) + ".GOF";
	fcFileName = filename.section(rxp3, 0, 0) + ".FCF";
    dim = dim.section(rxp2, 0, 0);
    if (dim.length() < 5)
    {
		return;
    }
    if (dim.contains("x") > 2)
    {
		dim = dim.section("x", 0, 2);
    }
    QString x = dim.section("x", 0, 0);
    QString y = dim.section("x", 1, 1);
    QString z = dim.section("x", 2, 2);
    
    dimension[0] = x.toInt(&ok, 10);
    dimension[1] = y.toInt(&ok, 10);
    dimension[2] = z.toInt(&ok, 10);
}


void viewWidgetImpl::readSpacing(QString filename)
{
    bool spacingOK = false;
    QRegExp rxp1 = QRegExp("_[0-9|.]+-[0-9|.]+-[0-9|.]+", TRUE, FALSE);
    QString spacingString = filename.section(rxp1, 1, 1, QString::SectionIncludeLeadingSep);
    spacingString = spacingString.section("_", -1, -1); 
    if (!spacingString.isEmpty())
    {
		QRegExp rxp2 = QRegExp("(.raw|.dat|.RAW|.DAT)", TRUE, FALSE);
		spacingString = spacingString.section(rxp2, 0, 0);
		if (spacingString.contains("-") == 2)
		{
			bool flag = true;
			spacing[0] = spacingString.section("-", 0, 0).toFloat(&flag);
			spacing[1] = spacingString.section("-", 1, 1).toFloat(&flag);
			spacing[2] = spacingString.section("-", 2, 2).toFloat(&flag);
			if (flag == true)
			{
				spacingOK = true;
				//cout << spacing[0] << spacing[1] << spacing[2] << "\n";
			}
		}
    }

    if (!spacingOK)
    {
		spacing[0] = 1;
		spacing[1] = 1;
		spacing[2] = 1;
    }
}


void viewWidgetImpl::defaultValues()
{
	// reset Statusvariablen
	interactive = false;
    xangle = 0;
    yangle = 0;
    zangle = 0;
	xstatus = 0;
	ystatus = 0;
	zstatus = 0;
	zoom = 1;
    zoomvalue = 1;
	parallelViewStatus = false;
	perspectiveViewStatus = true;
	sampleDistance = 1;

	dith_distance = 1;
	dith_size = 1;
	dith_context = true;
	dith_structure = true;
	dith_second_row = true;
	dith_second_row_structure = false;

	ML_direction[0] = 0;
	ML_direction[1] = 0;
	ML_direction[2] = 1;
	ML_angle = 15;
	ML_transferF = 0;
	ML_backside = false;
	ML_sideDir = 0;
	ML_uprightDir = 0;

	status_changed = false;

	FC_changes = false;
	DOF_changes = false;
	ML_changes = false;
	Dith_changes = false;

	DOF_randomSP_status = false;

	// reset Sliders and CheckBoxes
    xSlider->setValue(0);
    ySlider->setValue(0);
    zSlider->setValue(0);
    zoomSlider->setValue(100);
	sampleStepSize->setText("1");
	sampleStepSize->repaint();
	parallelView->setChecked(false);
	perspectiveView->setChecked(true);
	setFC(false);
	setDOF(false);

	Dith_distanceSlider->setValue(100);
	Dith_sizeSlider->setValue(100);
	Dith_context->setChecked(true);
	Dith_structure->setChecked(true);
	Dith_second_row->setChecked(true);
	Dith_second_row_structure->setChecked(false);
	Dith_second_row_structure->setEnabled(true);

	ML_angleDiffSlider->setValue(15);
	ML_transferFSlider->setValue(0);
	ML_backsideCB->setChecked(false);

	DOF_General->setChecked(false);
	DOF_DOF->setChecked(false);
	DOF_FC->setChecked(false);
	DOF_Dithering->setChecked(false);
	DOF_ML->setChecked(false);
	FC_General->setChecked(false);
	FC_DOF->setChecked(false);
	FC_FC->setChecked(false);
	FC_Dithering->setChecked(false);
	FC_ML->setChecked(false);
	Dith_General->setChecked(false);
	Dith_DOF->setChecked(false);
	Dith_FC->setChecked(false);
	Dith_Dithering->setChecked(false);
	Dith_ML->setChecked(false);
	ML_General->setChecked(false);
	ML_DOF->setChecked(false);
	ML_FC->setChecked(false);
	ML_Dithering->setChecked(false);
	ML_ML->setChecked(false);

	renewValues();
}


void viewWidgetImpl::setFCValues(QColor color, float transC, float ksc, float kss, float kse, float redOffset, float redFactor)
{
	FC_color_status = color;
	FC_transferCoefficient_status = transC;
	FC_ksc_status = ksc;
	FC_kss_status = kss;
	FC_kse_status = kse;
	FC_redOffset_status = redOffset;
	FC_redFactor_status = redFactor;
	FC_transferCoefficient->setText(QString::number(transC));
	FC_ksc->setText(QString::number(ksc));
	FC_kss->setText(QString::number(kss));
	FC_kse->setText(QString::number(kse));
	FC_redOffset->setText(QString::number(redOffset));
	FC_redFactor->setText(QString::number(redFactor));
}


vtkColorTransferFunction* viewWidgetImpl::colorFunction()
{
    vtkColorTransferFunction *function = vtkColorTransferFunction::New();
    if (transFunc->tf.length == 0)
    {
		function->AddRGBPoint (0, 255, 255, 255);
    }
    else
    {
		transfer_function tf = transFunc->tf;
		float *points = new float [4*tf.length];
		QColor *color = new QColor [tf.length];
		float *opacity = new float [tf.length];
		for (int i = 0; i < tf.length; i++)
		{
			points[4*i] = tf.element[i].begin - 45*tf.element[i].opacity*tan(tf.element[i].angle_left*PI/180);
			points[4*i+1] = tf.element[i].begin;
			points[4*i+2] = tf.element[i].begin + tf.element[i].length;
			points[4*i+3] = tf.element[i].begin + tf.element[i].length + 45*tf.element[i].opacity*tan(tf.element[i].angle_right*PI/180);
			color[i] = tf.element[i].color;
			opacity[i] = tf.element[i].opacity;
			points[4*i] /= factor;
			points[4*i+1] /= factor;
			points[4*i+2] /= factor;
			points[4*i+3] /= factor; 
			/*x = x/factor;
			int r = tf.element[i].color.red();
			int g = tf.element[i].color.green();
			int b = tf.element[i].color.blue();
			function->AddRGBPoint (x, (float)r/256, (float)g/256, (float)b/256);*/
		}
		for (i = 0; i < tf.length; i++)
		{
			float r1, g1, b1;
			r1 = (float)color[i].red()/256;
			g1 = (float)color[i].green()/256;
			b1 = (float)color[i].blue()/256;
			function->AddRGBPoint(points[4*i], 0, 0, 0);
			function->AddRGBPoint(points[4*i+3], 0, 0, 0);
			function->AddRGBPoint(points[4*i+1], r1, g1, b1);
			function->AddRGBPoint(points[4*i+2], r1, g1, b1);
			for (int j = 0; j < tf.length; j++)
			{
				if (j != i)
				{
					float r2, g2, b2;
					r2 = (float)color[j].red()/256;
					g2 = (float)color[j].green()/256;
					b2 = (float)color[j].blue()/256;

					// Point 1
					if (points[4*j] < points[4*i] &&
						points[4*j+3] > points[4*i])
					{
						function->AddRGBPoint(points[4*i], r2, g2, b2);
					}

					// Point 4
					if (points[4*j] < points[4*i+3] &&
						points[4*j+3] > points[4*i+3])
					{
						function->AddRGBPoint(points[4*i+3], r2, g2, b2);
					}

					// Point 2
					if (points[4*j] < points[4*i+1] &&
						points[4*j+1] > points[4*i+1])
					{
						float length = points[4*j+1] - points[4*j];
						float pos = points[4*i+1] - points[4*j];
						float opacity1 = opacity[i];
						float opacity2 = opacity[j]*pos/length;
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+1], r, g, b);
					}
					else if (points[4*j+1] <= points[4*i+1] &&
						points[4*j+2] >= points[4*i+1])
					{
						float opacity1 = opacity[i];
						float opacity2 = opacity[j];
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+1], r, g, b);
					}
					else if (points[4*j+2] < points[4*i+1] &&
						points[4*j+3] > points[4*i+1])
					{
						float length = points[4*j+3] - points[4*j+2];
						float pos = points[4*j+3] - points[4*i+1];
						float opacity1 = opacity[i];
						float opacity2 = opacity[j]*pos/length;
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+1], r, g, b);
					}

					// Point 3
					if (points[4*j] < points[4*i+2] &&
						points[4*j+1] > points[4*i+2])
					{
						float length = points[4*j+1] - points[4*j];
						float pos = points[4*i+1] - points[4*j];
						float opacity1 = opacity[i];
						float opacity2 = opacity[j]*pos/length;
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+2], r, g, b);
					}
					else if (points[4*j+1] <= points[4*i+2] &&
						points[4*j+2] >= points[4*i+2])
					{
						float opacity1 = opacity[i];
						float opacity2 = opacity[j];
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+2], r, g, b);
					}
					else if (points[4*j+2] < points[4*i+2] &&
						points[4*j+3] > points[4*i+2])
					{
						float length = points[4*j+3] - points[4*j+2];
						float pos = points[4*j+3] - points[4*i+1];
						float opacity1 = opacity[i];
						float opacity2 = opacity[j]*pos/length;
						float op_factor = opacity1 + opacity2;
						opacity1 /= op_factor;
						opacity2 /= op_factor;
						float r, g, b;
						r = opacity1*r1 + opacity2*r2;
						g = opacity1*g1 + opacity2*g2;
						b = opacity1*b1 + opacity2*b2;
						function->AddRGBPoint(points[4*i+2], r, g, b);
					}
				}
			}
		}
    }
    return function;
}


vtkPiecewiseFunction* viewWidgetImpl::opacityFunction ()
{
    vtkPiecewiseFunction *function = vtkPiecewiseFunction::New();
    if (transFunc->tf.length == 0)
    {
		function->AddSegment (1, 1.0, 256/factor, 1.0);
    }
    else
    {
		transfer_function tf = transFunc->tf;
		for (int i = 0; i < tf.length; i++)
		{
			float opacity = tf.element[i].opacity;
			float begin = tf.element[i].begin-45*tf.element[i].opacity*tan(tf.element[i].angle_left*PI/180);
			begin = begin/factor;
			float first = tf.element[i].begin/factor;
			float second = (tf.element[i].begin + tf.element[i].length) / factor;
			float end = second + (45*tf.element[i].opacity*tan (tf.element[i].angle_right*PI/180)) / factor;
			if (tf.element[i].angle_left != 0)
			{
				function->AddSegment (begin, 0, first, opacity);
			}
			else
			{
				function->AddSegment (first-1, 0, first, 0);
			}
			function->AddSegment (first, opacity, second, opacity);
			if (tf.element[i].angle_right != 0)
			{
				function->AddSegment (second, opacity, end, 0);
			}
			else
			{
				function->AddSegment(second, 0, second+1, 0);
			}
		}
    }
    return function;
}


void viewWidgetImpl::setEnabledWidgets(bool value)
{
    backgroundColorButton->setEnabled(value);
	setFocusButton->setEnabled(value);
	DOFAnimationButton->setEnabled(value);
	FCAnimationButton->setEnabled(value);
	renderButtonGeneral->setEnabled(value);
	renderButtonDOF->setEnabled(value);
	setFocusContextButton->setEnabled(value);
	renderButtonFC->setEnabled(value);
	loadGFButton->setEnabled(value);
	saveGFButton->setEnabled(value);
	loadFCButton->setEnabled(value);
	saveFCButton->setEnabled(value);
	contextColorButton->setEnabled(value);
	perspectiveView->setEnabled(value);
	parallelView->setEnabled(value);
	sampleStepSize->setEnabled(value);
	FC_transferCoefficient->setEnabled(value);
	FC_ksc->setEnabled(value);
	FC_kss->setEnabled(value);
	FC_kse->setEnabled(value);
	FC_redOffset->setEnabled(value);
	FC_redFactor->setEnabled(value);
	lenseSlider->setEnabled(value);
	samplePointsSlider->setEnabled(value);
	randomSamplePoints->setEnabled(value);
	sampleStepSize->setEnabled(value);
	perspectiveView->setEnabled(value);
	parallelView->setEnabled(value);
	contextOTFViewer->setEnabled(value);
	gradientViewer->setEnabled(value);
	Dith_distanceSlider->setEnabled(value);
	Dith_sizeSlider->setEnabled(value);
	Dith_context->setEnabled(value);
	Dith_structure->setEnabled(value);
	Dith_second_row->setEnabled(value);
	Dith_second_row_structure->setEnabled(value);
	renderButtonDith->setEnabled(value);
	renderButtonML->setEnabled(value);
	DithAnimationButton->setEnabled(value);
	MLAnimationButton->setEnabled(value);
	ML_directionButton->setEnabled(value);
	ML_angleDiffSlider->setEnabled(value);
	ML_transferFSlider->setEnabled(value);
	ML_backsideCB->setEnabled(value);
}


void viewWidgetImpl::renewValues()
{
    QString xrot = QString::number(xSlider->value()) + "";
    xRotLabel->setText(xrot);
	xRotLabel->repaint();
    QString yrot = QString::number(ySlider->value()) + "";
    yRotLabel->setText(yrot);
	yRotLabel->repaint();
    QString zrot = QString::number(zSlider->value()) + "";
    zRotLabel->setText(zrot);
	zRotLabel->repaint();
    QString zoom = QString::number((float)zoomSlider->value()/100);
    zoomLabel->setText(zoom);
	zoomLabel->repaint();
    QString lense = QString::number((float)lenseSlider->value()/10);
    lenseLabel->setText(lense);
	lenseLabel->repaint();
    QString sample = QString::number(samplePointsSlider->value());
    samplePointsLabel->setText(sample);
	samplePointsLabel->repaint();

	QString distance = QString::number((float)Dith_distanceSlider->value()/100);
    Dith_distanceLabel->setText(distance);
	Dith_distanceLabel->repaint();
    QString size = QString::number((float)Dith_sizeSlider->value()/100);
    Dith_sizeLabel->setText(size);
	Dith_sizeLabel->repaint();

	QString angle = QString::number(ML_angleDiffSlider->value());
    ML_angleDiffLabel->setText(angle);
	ML_angleDiffLabel->repaint();
    QString transferF = QString::number((float)ML_transferFSlider->value()/100);
    ML_transferFLabel->setText(transferF);
	ML_transferFLabel->repaint();
}


void viewWidgetImpl::saveImage()
{
    if (file_loaded)
    {
		QString saveFile = QFileDialog::getSaveFileName(
                    "",
                    "TIFF Image File (*.tif)",
                    this,
                    "save image dialog"
                    "Save Image" );
		if (!saveFile.isEmpty())
		{
			if (!saveFile.endsWith(".tif", false))
				saveFile = saveFile + ".tif";
			VTKview->saveImage(saveFile, DOF_status, randomSamplePoints->isChecked());
		}
    }
    else
    {
		QMessageBox::warning(this, "Depth of Field - Renderer", "No Image to save.");
    }
}


void viewWidgetImpl::DOFAnimation()
{
    int new_dim [3];
    new_dim [0] = (int)(dimension [0] * spacing [0]);
    new_dim [1] = (int)(dimension [1] * spacing [1]);
    new_dim [2] = (int)(dimension [2] * spacing [2]);
    //cout << new_dim[0] << "  y: " << new_dim[1] << "  z: "<< new_dim[2] << "\n";
    DOFAnimationEditorImpl *animation = new DOFAnimationEditorImpl(this, "Renderer - Animation Editor", new_dim, VTKview->getOrientation(), VTKview->getFocalPoint());
    animation->exec();
    if (animation->rendering)
    {

		QString directory = QFileDialog::getExistingDirectory(
                    "/home",
                    this,
                    "get existing directory",
                    "Choose a directory",
                    TRUE );
		if (!directory.isEmpty())
		{
			VTKview->startDOFAnimation(animation->focalArray, animation->numberPoints, animation->getStart(), animation->getEnd(), animation->getFrames(), animation->getSamplePoints(), animation->getFixedPoint(), directory);
		}
    }
	delete animation;
}


void viewWidgetImpl::setRenderStatus(QString status)
{
	renderStatus->setText(status);
	renderStatus->repaint();
}


void viewWidgetImpl::setFileStatus(QString status)
{
	fileStatus->setText(status);
	fileStatus->repaint();
}


void viewWidgetImpl::parallelViewClicked(bool status)
{
	if (status == true)
	{
		perspectiveView->setChecked(false);
	}
}


void viewWidgetImpl::perspectiveViewClicked(bool status)
{
	if (status == true)
	{
		parallelView->setChecked(false);
	}
}


void viewWidgetImpl::xRotationChanged(int degrees)
{
	renewValues();
    if (file_loaded && interactive)
    {
		int x = degrees - xstatus;
		VTKview->rendering(x, ystatus, zstatus, zoom, sampleDistance, parallelViewStatus);
		xstatus = degrees;
    }
    else if (file_loaded && !interactive)
    {
		xangle = degrees;
    }
}


void viewWidgetImpl::yRotationChanged(int degrees)
{
	renewValues();
    if (file_loaded && interactive)
    {
		int y = degrees - ystatus;
		VTKview->rendering(xstatus, y, zstatus, zoom, sampleDistance, parallelViewStatus);
		ystatus = degrees;
    }
    else if (file_loaded && !interactive)
    {
		yangle = degrees;
    }
}


void viewWidgetImpl::zRotationChanged(int degrees)
{
	renewValues();
    if (file_loaded && interactive)
    {
		int z = degrees - zstatus;
		VTKview->rendering(xstatus, ystatus, z, zoom, sampleDistance, parallelViewStatus);
		zstatus = degrees;
    }
    else if (file_loaded && !interactive)
    {
		zangle = degrees;
    }
}


void viewWidgetImpl::zoomChanged(int value)
{
	renewValues();
    if (file_loaded && interactive)
    {
		double z = (double)value/100;
		zoom = z;
		VTKview->rendering(xstatus, ystatus, zstatus, z, sampleDistance, parallelViewStatus);
    }
    else if (file_loaded && !interactive)
    {
		zoomvalue = (double)value/100;
    }
}


void viewWidgetImpl::backgroundColorButtonClicked()
{
	bool flag = false;
	QColor color = QColorDialog::getRgba (QColor((int)(background[0]*256), (int)(background[1]*256), (int)(background[2]*256)).rgb(), &flag, this, "Depth of Field - Color Chooser");
    if (flag)
    {
		background[0] = (float)color.red()/256;
		background[1] = (float)color.green()/256;
		background[2] = (float)color.blue()/256;
		background_changed = true;
    }
}


void viewWidgetImpl::renderButtonClicked()
{
	QString stepSize = sampleStepSize->text();
	bool rendering = false;
	bool flag = true;
	float distance = stepSize.toFloat(&flag);
	if (!flag)
	{
		tabWidget->setCurrentPage(0);
		QMessageBox::warning(this, "Depth of Field - Renderer", "Sample Step Size is no Float Value.");
		sampleStepSize->setText(QString::number(sampleDistance));
		sampleStepSize->repaint();
		distance = sampleDistance;
	}
	if (perspectiveViewStatus != perspectiveView->isChecked() || parallelViewStatus != parallelView->isChecked() || sampleDistance != distance)
	{
		perspectiveViewStatus = perspectiveView->isChecked();
		parallelViewStatus = parallelView->isChecked();
		sampleDistance = distance;
		VTKview->rendering(0, 0, 0, zoom, sampleDistance, parallelViewStatus);
		rendering = true;
	}
	if (gradientViewer->getChanged())
	{
		vtkPiecewiseFunction *gradientFunc = vtkPiecewiseFunction::New();
		gradientViewer->getGradientFunction(gradientFunc, VTKview->getGradientRange());
		VTKview->rendering(gradientFunc);
		rendering = true;
		gradientViewer->resetChanged();
	}
	if (background_changed)
	{
		VTKview->setBackground(background);
		background_changed = false;
		rendering = true;
	}
	if (xangle != xstatus || yangle != ystatus || zangle != zstatus || zoomvalue != zoom)
    {
		int x = xangle - xstatus;
		int y = yangle - ystatus;
		int z = zangle - zstatus;
		xstatus = xangle;
		ystatus = yangle;
		zstatus = zangle;
		zoom = zoomvalue;
		VTKview->rendering(x, y, z, zoomvalue, sampleDistance, parallelViewStatus);
		rendering = true;
    }

	if (FC_status)
	{
		bool flag = getFCValues();
		if (FC_changes || contextOTFViewer->getChanged())
		{
			vtkPiecewiseFunction *pwFunc = vtkPiecewiseFunction::New();
			contextOTFViewer->getOpacityFunction(pwFunc, range);
			vtkColorTransferFunction *cTF = vtkColorTransferFunction::New();
			cTF->AddRGBPoint(0, (float)FC_color_status.red()/256, (float)FC_color_status.green()/256, (float)FC_color_status.blue()/256);
			cTF->AddRGBPoint(range, (float)FC_color_status.red()/256, (float)FC_color_status.green()/256, (float)FC_color_status.blue()/256);
			VTKview->renderingFC(cTF, pwFunc, FC_transferCoefficient_status, FC_ksc_status, FC_kss_status, FC_kse_status, FC_redOffset_status, FC_redFactor_status);
			rendering = true;
			FC_changes = false;
			contextOTFViewer->resetChanged();
		}
	}

	if (Dith_status)
	{
		if (Dith_changes || (dith_context != Dith_context->isChecked()) || (dith_structure != Dith_structure->isChecked())
			|| (dith_second_row != Dith_second_row->isChecked()) || (dith_second_row_structure != Dith_second_row_structure->isChecked() && Dith_second_row_structure->isEnabled()))
		{
			Dith_changes = false;
			dith_context = Dith_context->isChecked();
			dith_structure = Dith_structure->isChecked();
			dith_second_row = Dith_second_row->isChecked();
			bool help = false;
			if (Dith_second_row_structure->isEnabled())
			{
				dith_second_row_structure = Dith_second_row_structure->isChecked();
				help = dith_second_row_structure;
			}
			VTKview->setDitheringParameters(dith_distance, dith_size, dith_context, dith_structure, dith_second_row, help);
			rendering = true;
		}
	}

	if (ML_status)
	{
		if (ML_changes || (ML_backside != ML_backsideCB->isChecked()))
		{
			ML_changes = false;
			ML_backside = ML_backsideCB->isChecked();
			VTKview->setMagicLampParameters(ML_direction, ML_angle, ML_transferF, ML_backside);
			rendering = true;
		}
	}

	if (rendering)
	{
		if (DOF_status)
			VTKview->renderingDOF(randomSamplePoints->isChecked());
		else
			VTKview->RenderImage();
	}
	else if (DOF_status)
		VTKview->renderingDOF(randomSamplePoints->isChecked());
	else if (status_changed && !DOF_status)
	{
		VTKview->RenderImage();
		status_changed = false;
	}
}


void viewWidgetImpl::lenseSliderChanged(int value)
{
	renewValues();
    double lense = (double)value/10;
	VTKview->setFocalDisk(lense);
	DOF_changes = true;
}


void viewWidgetImpl::samplePointsSliderChanged(int value)
{
	renewValues();
	VTKview->setSamplePoints(value);
	DOF_changes = true;
}


void viewWidgetImpl::setFocusContextButtonClicked()
{
	int new_dim [3];
    new_dim [0] = (int)(dimension [0] * spacing [0]);
    new_dim [1] = (int)(dimension [1] * spacing [1]);
    new_dim [2] = (int)(dimension [2] * spacing [2]);
    focusContextEditorImpl *focConEditor = new focusContextEditorImpl(this, "Renderer - Set Focus Region", new_dim, VTKview->getOrientation(), VTKview->getFocalPoint(), FC_middle, FC_radius);
    focConEditor->exec();
    if (focConEditor->okay)
    {
		float *middle;
		float radius;
		middle = focConEditor->getFocalPoint();
		radius = focConEditor->getRadius();
		if (middle[0] != FC_middle[0] ||
			middle[1] != FC_middle[1] ||
			middle[2] != FC_middle[2] ||
			radius != FC_radius)
		{
			FC_middle[0] = middle[0];
			FC_middle[1] = middle[1];
			FC_middle[2] = middle[2];
			FC_radius = radius;
			VTKview->renderingFC(FC_middle, FC_radius);
			if (FC_status)
				VTKview->RenderImage();
		}
    }
	delete focConEditor;
}


void viewWidgetImpl::loadGFButtonClicked()
{
	QString openFile = QFileDialog::getOpenFileName(
                    goFileName,
                    "GradientF-F (*.gof)",
                    this,
                    "open file dialog"
                    "Choose a file to Open" );
    if (!openFile.isEmpty())
	{
		GOReader *go_reader = new GOReader(openFile);
		if (!go_reader->read())
			QMessageBox::warning(this, "Depth of Field - Renderer", "General: Error while reading GradientO-File.");
		else
		{
			gradientViewer->setGradientFunction(go_reader->getPiecewiseFunction(), range);
			vtkPiecewiseFunction *gradientFunc = vtkPiecewiseFunction::New();
			gradientViewer->getGradientFunction(gradientFunc, range);
			VTKview->rendering(gradientFunc);
		}
		delete go_reader;
	}
}


void viewWidgetImpl::saveGFButtonClicked()
{
	QString saveFile = QFileDialog::getSaveFileName(
                    goFileName,
                    "GradientF-F (*.gof)",
                    this,
                    "save file dialog"
                    "Choose a filename to save under" );
	if (!saveFile.isEmpty())
	{
		if (!saveFile.endsWith(".gof", false))
				saveFile = saveFile + ".gof";
		vtkPiecewiseFunction *pwFunction = vtkPiecewiseFunction::New();
		gradientViewer->getGradientFunction(pwFunction, range);
		GOWriter *go_writer = new GOWriter(saveFile);
		go_writer->write(pwFunction);
		delete go_writer;
	}
}


void viewWidgetImpl::loadFCButtonClicked()
{
	QString openFile = QFileDialog::getOpenFileName(
                    fcFileName,
                    "FocusContext-F (*.fcf)",
                    this,
                    "open file dialog"
                    "Choose a file to Open" );
    if (!openFile.isEmpty())
	{
		FCReader *fc_reader = new FCReader(openFile);
		if (!fc_reader->read())
		{
			QMessageBox::warning(this, "Depth of Field - Renderer", "Focus + Context: Error while reading F+C-File.");
		}
		else
		{
			float ksc, kss, kse, transC, redOffset, redFactor;
			QColor color;
			vtkPiecewiseFunction *pwFunc = vtkPiecewiseFunction::New();
			fc_reader->getFCSettings(pwFunc, &color, &transC, &ksc, &kss, &kse, &redOffset, &redFactor);
			contextOTFViewer->setOpacityFunction(pwFunc, range);
			setFCValues(color, transC, ksc, kss, kse, redOffset, redFactor);
			vtkColorTransferFunction *cTF = vtkColorTransferFunction::New();
			cTF->AddRGBPoint(0, color.red()/256, color.green()/256, color.blue()/256);
			cTF->AddRGBPoint(range, color.red()/256, color.green()/256, color.blue()/256);
			VTKview->renderingFC(cTF, pwFunc, transC, ksc, kss, kse, redOffset, redFactor);
			FC_changes = true;
		}
		delete fc_reader;
	}
}


void viewWidgetImpl::saveFCButtonClicked()
{
	if (getFCValues())
	{
		QString saveFile = QFileDialog::getSaveFileName(
			            fcFileName,
				        "FocusContext-F (*.fcf)",
					    this,
						"save file dialog"
						"Choose a filename to save under" );
		if (!saveFile.isEmpty())
		{
			if (!saveFile.endsWith(".fcf", false))
				saveFile = saveFile + ".fcf";
			vtkPiecewiseFunction *pwFunction = vtkPiecewiseFunction::New();
			contextOTFViewer->getOpacityFunction(pwFunction, range);
			FCWriter *fc_writer = new FCWriter(saveFile);
			fc_writer->write(pwFunction, FC_color_status, FC_transferCoefficient_status, FC_ksc_status, FC_kss_status, FC_kse_status, FC_redOffset_status, FC_redFactor_status);
			delete fc_writer;
		}
	}
}


bool viewWidgetImpl::getFCValues()
{
	bool flag1, flag2, flag3, flag4, flag5, flag6;
	float tC = (FC_transferCoefficient->text()).toFloat(&flag1);
	float k_sc = (FC_ksc->text()).toFloat(&flag2);
	float k_ss = (FC_kss->text()).toFloat(&flag3);
	float k_se = (FC_kse->text()).toFloat(&flag4);
	float redOffset = (FC_redOffset->text()).toFloat(&flag5);
	float redFactor = (FC_redFactor->text()).toFloat(&flag6);
	if (!flag1 || !flag2 || !flag3 || !flag4 || !flag5 || !flag6)
	{
		tabWidget->setCurrentPage(2);
		QMessageBox::warning(this, "Depth of Field - Renderer", "Focus + Context: Please fill in float Values.");
		if (!flag1)
		{
			FC_transferCoefficient->setText(QString::number(FC_transferCoefficient_status));
			FC_transferCoefficient->repaint();
		}
		if (!flag2)
		{
			FC_ksc->setText(QString::number(FC_ksc_status));
			FC_ksc->repaint();
		}
		if (!flag3)
		{
			FC_kss->setText(QString::number(FC_kss_status));
			FC_kss->repaint();
		}
		if (!flag4)
		{
			FC_kse->setText(QString::number(FC_kse_status));
			FC_kse->repaint();
		}
		if (!flag5)
		{
			FC_redOffset->setText(QString::number(FC_redOffset_status));
			FC_redOffset->repaint();
		}
		if (!flag6)
		{
			FC_redFactor->setText(QString::number(FC_redFactor_status));
			FC_redFactor->repaint();
		}
		return false;
	}
	else
	{
		if (FC_transferCoefficient_status != tC ||
			FC_ksc_status != k_sc ||
			FC_kss_status != k_ss ||
			FC_kse_status != k_se ||
			FC_redOffset_status != redOffset ||
			FC_redFactor_status != redFactor)
		{
			FC_transferCoefficient_status = tC;
			FC_ksc_status = k_sc;
			FC_kss_status = k_ss;
			FC_kse_status = k_se;
			FC_redOffset_status = redOffset;
			FC_redFactor_status = redFactor;
			FC_changes = true;
		}
		return true;
	}
}


void viewWidgetImpl::setDOF(bool status)
{
	DOF_General->setChecked(status);
	DOF_DOF->setChecked(status);
	DOF_FC->setChecked(status);
	DOF_Dithering->setChecked(status);
	DOF_ML->setChecked(status);

	DOF_status = status;
}


void viewWidgetImpl::setFC(bool status)
{
	FC_General->setChecked(status);
	FC_DOF->setChecked(status);
	FC_FC->setChecked(status);
	FC_Dithering->setChecked(status);
	FC_ML->setChecked(status);

	FC_status = status;

	VTKview->setFocusContext(status);
	status_changed = true;
}


void viewWidgetImpl::setDith(bool status)
{
	Dith_General->setChecked(status);
	Dith_DOF->setChecked(status);
	Dith_FC->setChecked(status);
	Dith_Dithering->setChecked(status);
	Dith_ML->setChecked(status);

	Dith_status = status;

	VTKview->setDithering(status);
	status_changed = true;
}


void viewWidgetImpl::setML(bool status)
{
	ML_General->setChecked(status);
	ML_DOF->setChecked(status);
	ML_FC->setChecked(status);
	ML_Dithering->setChecked(status);
	ML_ML->setChecked(status);

	ML_status = status;

	VTKview->setMagicLamp(status);
	status_changed = true;
}


void viewWidgetImpl::contextColorButtonClicked()
{
    bool flag = false;
    QColor color = QColorDialog::getRgba (FC_color_status.rgb(), &flag, this, "Depth of Field - Color Chooser");
    if (flag == true)
    {
		FC_color_status = color;
		FC_changes = true;
    }
	status_changed = true;
}


void viewWidgetImpl::FCAnimation()
{
	int new_dim [3];
    new_dim [0] = (int)(dimension [0] * spacing [0]);
    new_dim [1] = (int)(dimension [1] * spacing [1]);
    new_dim [2] = (int)(dimension [2] * spacing [2]);
	FCAnim_struct startPoint;
	startPoint.center[0] = FC_middle[0];
	startPoint.center[1] = FC_middle[1];
	startPoint.center[2] = FC_middle[2];
	startPoint.radius = FC_radius;
	double * focPoint = VTKview->getFocalPoint();
	startPoint.focPoint[0] = focPoint[0];
	startPoint.focPoint[1] = focPoint[1];
	startPoint.focPoint[2] = focPoint[2];
	startPoint.dimension[0] = new_dim[0];
	startPoint.dimension[1] = new_dim[1];
	startPoint.dimension[2] = new_dim[2];
	startPoint.xRot = xstatus;
	startPoint.xClockwise = true;
	startPoint.yRot = ystatus;
	startPoint.yClockwise = true;
	startPoint.zRot = zstatus;
	startPoint.zClockwise = true;
	startPoint.zoom = zoom;
	startPoint.transfC = FC_transferCoefficient_status;
	startPoint.ksc = FC_ksc_status;
	startPoint.kss = FC_kss_status;
	startPoint.kse = FC_kse_status;
	startPoint.redOffset = FC_redOffset_status;
	startPoint.redFactor = FC_redFactor_status;
	startPoint.samples = 2;
    FCAnimationEditorImpl *animation = new FCAnimationEditorImpl(this, "Renderer - FC Animation Editor", startPoint);
    animation->exec();
    if (animation->getExitStatus())
    {

		QString directory = QFileDialog::getExistingDirectory(
                    "/home",
                    this,
                    "get existing directory",
                    "Choose a directory",
                    TRUE );
		if (!directory.isEmpty())
		{
			FCAnim_struct *animPoints = new FCAnim_struct [30];
			int number;
			animPoints = animation->getFCAnimStruct();
			number = animation->getNumbers();
			int i = 0;
			VTKview->startFCAnimation(animPoints, number, directory, FC_status);
		}
    }
	delete animation;
}


void viewWidgetImpl::DithAnimation()
{
	DithAnim_struct startPoint;
	startPoint.distance = dith_distance;
	startPoint.size = dith_size;
	startPoint.context = dith_context;
	startPoint.structure = dith_structure;
	startPoint.sec_row = dith_second_row;
	startPoint.sec_row_structure = dith_second_row_structure;
	startPoint.xRot = xstatus;
	startPoint.xClockwise = true;
	startPoint.yRot = ystatus;
	startPoint.yClockwise = true;
	startPoint.zRot = zstatus;
	startPoint.zClockwise = true;
	startPoint.zoom = zoom;
	startPoint.samples = 2;

	DithAnimationEditorImpl *animation = new DithAnimationEditorImpl(this, "Renderer - Dithering Animation Editor", startPoint);
    animation->exec();
    if (animation->getExitStatus())
    {

		QString directory = QFileDialog::getExistingDirectory(
                    "/home",
                    this,
                    "get existing directory",
                    "Choose a directory",
                    TRUE );
		if (!directory.isEmpty())
		{
			DithAnim_struct *animPoints = new DithAnim_struct [30];
			int number;
			animPoints = animation->getDithAnimStruct();
			number = animation->getNumbers();
			int i = 0;
			VTKview->startDithAnimation(animPoints, number, directory, Dith_status);
			bool help = false;
			if (Dith_second_row_structure->isEnabled())
			{
				dith_second_row_structure = Dith_second_row_structure->isChecked();
				help = dith_second_row_structure;
			}
			VTKview->setDitheringParameters(dith_distance, dith_size, dith_context, dith_structure, dith_second_row, help);
			VTKview->RenderImage();
		}
    }
	delete animation;
}


void viewWidgetImpl::MLAnimation()
{
	MLAnim_struct startPoint;
	startPoint.direction[0] = ML_direction[0];
	startPoint.direction[1] = ML_direction[1];
	startPoint.direction[2] = ML_direction[2];
	startPoint.angle_diff = ML_angle;
	startPoint.transfer = ML_transferF;
	startPoint.backside = ML_backside;
	startPoint.sideDir = ML_sideDir;
	startPoint.uprightDir = ML_uprightDir;
	startPoint.xRot = xstatus;
	startPoint.xClockwise = true;
	startPoint.yRot = ystatus;
	startPoint.yClockwise = true;
	startPoint.zRot = zstatus;
	startPoint.zClockwise = true;
	startPoint.zoom = zoom;
	startPoint.samples = 2;

	MLAnimationEditorImpl *animation = new MLAnimationEditorImpl(this, "Renderer - Magic Lamp Animation Editor", startPoint);
    animation->exec();
    if (animation->getExitStatus())
    {

		QString directory = QFileDialog::getExistingDirectory(
                    "/home",
                    this,
                    "get existing directory",
                    "Choose a directory",
                    TRUE );
		if (!directory.isEmpty())
		{
			MLAnim_struct *animPoints = new MLAnim_struct [30];
			int number;
			animPoints = animation->getMLAnimStruct();
			number = animation->getNumbers();
			int i = 0;
			VTKview->startMLAnimation(animPoints, number, directory, ML_status);
		}
    }
	delete animation;
}


void viewWidgetImpl::DithDistanceSliderChanged(int value)
{
	float distance = (float)value/100;
	if (distance != dith_distance)
	{
		Dith_distanceLabel->setText(QString::number(distance));
		Dith_changes = true;
		dith_distance = distance;
	}
}


void viewWidgetImpl::DithSizeSliderChanged(int value)
{
	float size = (float)value/100;
	if (size != dith_size)
	{
		Dith_sizeLabel->setText(QString::number(size));
		Dith_changes = true;
		dith_size = size;
	}
}


void viewWidgetImpl::DithSecondRowClicked(bool value)
{
	if (!value)
		Dith_second_row_structure->setEnabled(false);
	else
		Dith_second_row_structure->setEnabled(true);
}


void viewWidgetImpl::MLDirectionButtonClicked()
{
	MLDirectionEditorImpl *mlDirEditor = new MLDirectionEditorImpl(this, "MLDirEditor", ML_sideDir, ML_uprightDir);
	mlDirEditor->exec();
	if (mlDirEditor->getExitStatus())
	{
		float upright = mlDirEditor->getUprightDirection();
		float side = mlDirEditor->getSideDirection();
		ML_direction[2] = cos(upright*PI/180) * cos(side*PI/180);
		ML_direction[0] = (-1)*sin(upright*PI/180) * cos(side*PI/180);
		ML_direction[1] = (-1)*sin(side*PI/180);
		float length = sqrt(ML_direction[0]*ML_direction[0] + ML_direction[1]*ML_direction[1]+ ML_direction[2]*ML_direction[2]);
		ML_direction[0] /= length;
		ML_direction[1] /= length;
		ML_direction[2] /= length;
		ML_uprightDir = upright;
		ML_sideDir = side;
		ML_changes = true;
	}
}


void viewWidgetImpl::MLAngleSliderChanged(int value)
{
	float angle = (float)value;
	if (angle != ML_angle)
	{
		ML_angleDiffLabel->setText(QString::number(angle));
		ML_changes = true;
		ML_angle = angle;
	}
}


void viewWidgetImpl::MLTransferSliderChanged(int value)
{
	float transferF = (float)value/100;
	if (transferF != ML_transferF)
	{
		ML_transferFLabel->setText(QString::number(transferF));
		ML_changes = true;
		ML_transferF = transferF;
	}
}