/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkVolumeRayCastCompositeFunction.h,v $
  Language:  C++
  Date:      $Date: 2002/10/28 21:33:01 $
  Version:   $Revision: 1.29 $

  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

// VTK-Class
// modified by Martin Haidacher


#ifndef VolumeRayCastCompositeFunction_h
#define VolumeRayCastCompositeFunction_h

#include "vtkVolumeRayCastFunction.h"

#include "vtkColorTransferFunction.h"
#include "vtkPiecewiseFunction.h"

#define VTK_COMPOSITE_CLASSIFY_FIRST 0
#define VTK_COMPOSITE_INTERPOLATE_FIRST 1


//! Struct dataset
/*!
	Struktur, welche die Parameter f&uuml;r die verschiedenen Render-Methoden speichert.
*/
struct dataset
{
	float *foc_middle;
	float foc_radius;
	bool FOCUS_CONTEXT;
	bool DITHERING;
	bool DITHERING_CONTEXT;
	bool DITHERING_STRUCT;
	bool DITHERING_SECROW;
	bool DITHERING_SECROW_STRUCT;
	bool MAGIC_LAMP;
	bool TRI_INT;
	vtkColorTransferFunction *con_colorTF;
	vtkPiecewiseFunction * con_gradientTF;
	float k_sc;
	float k_ss;
	float k_se;
	float trans_factor;
	float reductionFactor;
	float reductionOffset;
	float d_distance;
	float d_size;
	float d_struct_color [3];
	float d_struct_opacity;
	float ml_direction [3];
	float ml_angle;
	bool ml_back_front;
	float ml_transfer;
};


//! Class VolumeRayCastCompositeFunction
/*!
    Diese Klasse stammt von VTK. Sie wurde von mir umgeschrieben. In ihr werden alle Methoden
	des Programms ausprogrammiert. Sie ist Teil des Raycasting-Verfahrens.
*/
class VolumeRayCastCompositeFunction : public vtkVolumeRayCastFunction
{
public:
	//! Konstruktor
	/*!
		Initialisierung der Parameter in der Struktur.
	*/
	VolumeRayCastCompositeFunction();
	//! Destruktor
	~VolumeRayCastCompositeFunction();
	//! Methode VolumeRayCastCompositeFunction
	/*!
		Alternative zum Konstruktor, Standard in VTK.
	*/
	static VolumeRayCastCompositeFunction *New();
	void PrintSelf( ostream& os, vtkIndent indent );
	//! Methode SetCompositeMethodToInterpolateFirst
	/*!
		Setzt die Interpolationsmethode auf Interpolation vor Klassifikation.
	*/
	void SetCompositeMethodToInterpolateFirst()
	{ this->CompositeMethod = VTK_COMPOSITE_INTERPOLATE_FIRST; }
	//! Methode SetCompositeMethodToClassifyFirst
	/*!
		Setzt die Interpolationsmethode auf Klassifikation vor Interpolation.
	*/
	void SetCompositeMethodToClassifyFirst() 
    { this->CompositeMethod = VTK_COMPOSITE_CLASSIFY_FIRST; }
	//! Methode GetCompositeMethodAsString
	/*!
		Gibt Kompositionsmethode als String zur&uuml;ck.
	*/
	const char *GetCompositeMethodAsString(void);

	//! Methode CastRay
	/*!
		Berechnet den Farbwert f&uuml;r einen Strahl. Hier findet der Hauptrendering-Schritt
		statt.
	*/
	void CastRay( vtkVolumeRayCastDynamicInfo *dynamicInfo,
		            vtkVolumeRayCastStaticInfo *staticInfo);

	//! Methode GetZeroOpacityThreshold
	/*!
		Gibt den Schwellwert f&uuml;r die Opacity zur&uuml;ck.
	*/
	float GetZeroOpacityThreshold( vtkVolume *vol );

	//! Methode setFocusContextEnable
	/*!
		Schaltet die Focus+Context Methode ein.
	*/
	void setFocusContextEnable();
	//! Methode setFocusContextDisable
	/*!
		Schaltet die Focus+Context Methode aus.
	*/
	void setFocusContextDisable();
	//! Methode getFocusContextStatus
	/*!
		Gibt zur&uuml;ck ob die Focus+Context Methode ein oder aus geschalten ist.
	*/
	bool getFocusContextStatus();
	//! Methode setFocusArea
	/*!
		Setzt den Fokusbereich beim Focus+Context.
	*/
	void setFocusArea(float *middle, float radius);
	//! Methode setContextVolume
	/*!
		Setzt die Transfer Function f&uuml;r den Kontextbereich.
	*/
	void setContextVolume(vtkColorTransferFunction *con_colorTF, vtkPiecewiseFunction * con_gradientTF);
	//! Methode setParameters
	/*!
		Setzt die Parameter f&uuml;r den Kontextbereich.
	*/
	void setParameters(float transC, float ksc, float kss, float kse, float reducOffset, float reducFac);


	//! Methode setDithering
	/*!
		Setzt die Parameter f&uuml;r das 3D Dithering.
	*/
	void setDithering(int dim [3], float spacing [3], float distance, float size, bool context, bool structure, bool second_row, bool second_row_structure);
	//! Methode setDithering
	/*!
		Setzt die Parameter f&uuml;r das 3D Dithering.
	*/
	void setDithering(bool context, bool structure, bool second_row, bool second_row_structure);
	//! Methode setDitheringEnable
	/*!
		Schaltet die Dithering Methode ein.
	*/
	void setDitheringEnable();
	//! Methode setDitheringDisable
	/*!
		Schaltet die Dithering Methode aus.
	*/
	void setDitheringDisable();
	//! Methode setDitheringDisable
	/*!
		Gibt zur&uuml;ck ob die Dithering Methode ein oder aus geschalten ist. 
	*/
	bool getDitheringStatus();

	//! Methode setMagicLamp
	/*!
		Setzt die Parameter f&uuml;r die Magic Lamp.
	*/
	void setMagicLamp(float direction [3], float angle, float transfer_factor, bool backside);
	//! Methode setMagicLampDirection
	/*!
		Setzt die Richtung aus der die Magic Lamp strahlt.
	*/
	void setMagicLampDirection(float direction [3]);
	//! Methode getMagicLampDirection
	/*!
		Gibt die Richtung aus der die Magic Lamp strahlt zur&uuml;ck.
	*/
	float * getMagicLampDirection();
	//! Methode setMagicLampEnable
	/*!
		Schaltet die Magic Lamp Methode ein.
	*/
	void setMagicLampEnable();
	//! Methode setMagicLampDisable
	/*!
		Schaltet die Magic Lamp Methode aus.
	*/
	void setMagicLampDisable();
	//! Methode getMagicLampStatus
	/*!
		Gibt zur&uuml;ck ob Magic Lamp ein- oder ausgeschalten ist.
	*/
	bool getMagicLampStatus();

protected:
	void SpecificFunctionInitialize( vtkRenderer *ren,
		                             vtkVolume   *vol,
			                         vtkVolumeRayCastStaticInfo *staticInfo,
									 vtkVolumeRayCastMapper *mapper );  
	int CompositeMethod;

private:
	void setPoints(char *array, int dim [3], float spacing [3], float distance, float size);
	bool setRegion(char *array, int dim [3], int point [3], float size [3], int value);
	bool setStruct(char *array, int dim [3], int point [3], float step [3], int value);
	bool isInside(int dim [3], int point [3]);

	bool context_volume_loaded, dithering_pattern_loaded;
	char *ditheringPattern;
	dataset datas;
};


#endif
