#include "dofmainwindow.h"
#include <qapplication.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qframe.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qregexp.h>
#include <qlayout.h>
#include <qcolor.h>

#include "dimdialog.h"
#include "reader.h"
#include "transferFunction.h"
#include "tfReader.h"


dofMainWindow::dofMainWindow (QWidget* parent, const char* name, WFlags f) : QWidget (parent, name, f)
{
    QAction *fileOpenAction;
    QAction *imageSaveAction;
    
    fileOpenAction = new QAction(
	    "Open File", QPixmap("icons/file_open64x64.png"),
	    "&Open...", CTRL+Key_O, this, "open" );
    
    imageSaveAction = new QAction(
	    "Save Image", "&Save Image...", CTRL+Key_S, this, "save");
    
    // Create a menu
    QPopupMenu *file = new QPopupMenu(this);
    fileOpenAction->addTo(file);
    imageSaveAction->addTo(file);
    file->insertSeparator();
    file->insertItem("Exit", qApp, SLOT(quit()), CTRL+Key_Q);

    // Create a menu bar
    QMenuBar *menu = new QMenuBar(this);
    menu->setSeparator(QMenuBar::InWindowsStyle);
    menu->insertItem("&File", file);

    // Create a nice frame to put around the OpenGL widget
    QFrame *frame = new QFrame(this, "frame");
    frame->setFrameStyle(QFrame::Plain | QFrame::NoFrame);
    frame->setLineWidth(2);
    
    viewWidget = new viewWidgetImpl(frame, "viewWidget");
    
    // Put the GL widget inside the frame
    QHBoxLayout* flayout = new QHBoxLayout(frame, 2, 2, "flayout");
    flayout->addWidget(viewWidget, 1);
    
    // Top level layout, puts the sliders to the left of the frame/GL widget
    QHBoxLayout* hlayout = new QHBoxLayout(this, 0, 0, "hlayout");
    hlayout->setMenuBar(menu);
    hlayout->addWidget(frame, 1);
    
    // connections
    connect(fileOpenAction, SIGNAL(activated()), viewWidget, SLOT(openFile()));
    connect(imageSaveAction, SIGNAL(activated()), viewWidget, SLOT(saveImage()));
}


dofMainWindow::~dofMainWindow ()
{
    delete viewWidget;
}
